@@ -167,6 +167,7 @@ void config_audio_device(int rate_set, int bits_set, int stereo_set)
 long playback_samples_remaining = 0 ;
 long playback_total_bytes ;
 int playback_bytes_per_block ;
+int looped_count ;
 
 #define MAXBUFSIZE 32768
 int BUFSIZE ;
@@ -183,7 +184,7 @@ long set_playback_cursor_position(struct view *v, long millisec_per_visual_frame
     extern int audio_playback ;
 
     if(audio_playback == TRUE) {
-	long bytes = audio_device_processed_bytes();
+	long bytes = audio_device_processed_bytes()-looped_count*playback_total_bytes ;
 	get_region_of_interest(&first, &last, v) ;
 
 	v->cursor_position = first_playback_sample+bytes/(PLAYBACK_FRAMESIZE) ;
@@ -269,6 +270,7 @@ long start_playback(char *output_device, struct view *v, struct sound_prefs *p,
 /*      g_print("BUFSIZE %ld (%lg fragments)\n", (long)BUFSIZE, (double)BUFSIZE/(double)oss_info.fragsize) ;  */
 
     v->prev_cursor_position = -1 ;
+    looped_count = 0 ;
 
     return playback_samples ;
 }
@@ -821,21 +823,30 @@ int process_audio(gfloat *pL, gfloat *pR)
 	n_read = sf_readf_int(sndfile, p_int, n_samples_to_read) ;
     }
 
+    if(playback_samples_remaining - n_read < 1) feather_out = 1 ;
+
     for(i = 0  ; i < n_read ; i++) {
 	int vl, vr ;
 
 	if(BYTESPERSAMPLE < 3) {
-	    if(feather_out == 1 && n_read-(i+1) < 10) {
-		double p = (double)(n_read-(i+1))/9.0 ;
+	    if(feather_out == 1 && n_read-(i+1) < 100) {
+		int j = (n_read-(i))-1 ;
+		double p = (double)(j)/99.0 ;
+		if(i > n_read - 100) {
+		    //printf("j:%d %lf %hd %hd ", j, p, p_short[i], p_short[i+1]) ;
+		}
 		p_short[i] *= p ;
 		p_short[i+1] *= p ;
-		printf(".") ;
+		if(i > n_read - 100) {
+		    //printf("%hd %hd\n", p_short[i], p_short[i+1]) ;
+		}
+		//if(i == n_read-1) printf(" %lf", p) ;
 	    }
 	    vl = p_short[i] ;
 	    vr = p_short[i+1] ;
 	} else {
-	    if(feather_out == 1 && n_read-(i+1) < 10) {
-		double p = (double)(n_read-(i+1))/9.0 ;
+	    if(feather_out == 1 && n_read-(i+1) < 10000) {
+		double p = 1.0 - (double)(n_read-(i+1))/9999.0 ;
 		printf(".") ;
 		p_int[i] *= p ;
 		p_int[i+1] *= p ;
@@ -869,17 +880,47 @@ int process_audio(gfloat *pL, gfloat *pR)
 	playback_position += n_read ;
 	playback_samples_remaining -= n_read ;
 	if(playback_samples_remaining < 1) {
-	    audio_state = PLAYBACK ;
-	    audio_playback = FALSE ;
-	    g_print("playback_samples_remaining:%ld\n", playback_samples_remaining) ;
-	    return 1 ;
+	    extern int audio_is_looping ;
+	    if(audio_is_looping == FALSE) {
+		audio_state = PLAYBACK ;
+		audio_playback = FALSE ;
+		g_print("Stop playback with playback_samples_remaining:%ld\n", playback_samples_remaining) ;
+		return 1 ;
+	    } else {
+		playback_position = playback_start_position ;
+		playback_samples_remaining = (playback_end_position-playback_start_position) ;
+		sf_seek(sndfile, playback_position, SEEK_SET) ;
+		g_print("Loop with playback_samples_remaining:%ld\n", playback_samples_remaining) ;
+		looped_count++ ;
+	    }
 	}
     }
     return 0 ;
 }
 
-void stop_playback(void)
+void stop_playback(int force)
 {
+    if(!force) {
+	/* Robert altered */
+	int new_playback = audio_device_processed_bytes();
+	int  old_playback;
+
+	while(new_playback < playback_total_bytes) {
+	    /* Robert altered */
+	    usleep(100) ;
+	    old_playback = new_playback;
+	    new_playback=audio_device_processed_bytes();
+
+	    /* check if more samples have been processed, if not,quit */
+	    if (old_playback==new_playback){
+		fprintf(stderr,"Playback appears frozen\n Breaking\n");
+		break;
+	    }
+	}
+
+	usleep(100) ;
+    }
+
     audio_state = NOTHING ;
     audio_device_close() ;
 }
