@@ -155,20 +155,25 @@ int lsar_sample_restore(fftw_real data[], int firstbad, int lastbad, int siglen)
     int i, j, rows, cols ;
     int rcode ;
     gboolean clipped ;
-    double x[60], auto_coefs[61] ;
+    double x[100], auto_coefs[101] ;
     static MAT *A=MNULL, *Au=MNULL, *Aut=MNULL, *AutmAu=MNULL, *iAutmAu=MNULL, *final=MNULL ;
 /*      static MAT *A, *Au, *Aut, *AutmAu, *iAutmAu, *final ;  */
     static VEC *rhs, *sig, *sig_final ;
 
+    //estimate_region(data, firstbad, lastbad, siglen) ;
+    //return REPAIR_SUCCESS ;
+
     autolen = (siglen-n_bad)/4 ;
+    //autolen *= 2 ;
 
     if(autolen < 0) {
 	d_print("Autolen < 0!\n") ;
 	return REPAIR_FAILURE;
     }
 
-    if(autolen > 60) autolen = 60 ;
-    d_print("siglen:%d n_bad:%d Autolen:%d\n",siglen,n_bad,autolen) ;
+    if(autolen > 3*n_bad) autolen = 3*n_bad ;
+    if(autolen > 100) autolen = 100 ;
+    //g_print("siglen:%d n_bad:%d Autolen:%d\n",siglen,n_bad,autolen) ;
 
     sig = v_get(siglen) ;
     A = m_resize(A,siglen-autolen, siglen) ;
@@ -278,6 +283,7 @@ int declick_a_click(struct sound_prefs *p, long first_sample, long last_sample,
     fftw_real estimated[FFT_MAX*3], window_coef[FFT_MAX] ;
     fftw_real data[2][FFT_MAX] ;
 
+
     /* choose a repair strategy based on the length of the click */
 
     if(n_samples < 1) {
@@ -291,10 +297,10 @@ int declick_a_click(struct sound_prefs *p, long first_sample, long last_sample,
 	if(last > p->n_samples-1) last = p->n_samples-1 ;
 	repair_method = DECLICK_CUBIC ;
     } else {
-	/* cubic function -- interpolation */
-	first = first_sample-120 ;
+	/* LSAR */
+	first = first_sample-200;
 	if(first < 0) first = 0 ;
-	last = last_sample+120 ;
+	last = last_sample+200;
 	if(last > p->n_samples-1) last = p->n_samples-1 ;
 	repair_method = DECLICK_LSAR ;
     }
@@ -451,6 +457,290 @@ void get_hpf (long sample, fftw_real channel_data[], double *hpf, double *hpf_av
 
 char *do_declick(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
 struct click_data *clicks, int iterate_flag)
+{
+    extern int declick_detector_type ;
+
+    if(declick_detector_type == FFT_DETECT)
+	return do_declick_fft(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag) ;
+    else
+	return do_declick_hpf(p,first_sample,last_sample,channel_mask,sensitivity,repair,clicks,iterate_flag) ;
+}
+
+#define DECLICK_MAX_FFT 128
+char *do_declick_fft(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
+struct click_data *clicks, int iterate_flag)
+{
+    static char results_buf[200] ;
+    long min_sample, max_sample ;
+    long sample ;
+    long window_first ;
+    long i,k ;
+    long n_samples = last_sample - first_sample + 1 ;
+    int FFT_SIZE = 64 ;
+    int n_repaired[2] , n_this_pass, n_last_pass;
+    int n_not_repaired[2] ;
+    int n ;
+    int offset0,offsetF;
+    char max_exceeded_notice = 0 ;
+#define FFT_WINDOW 1000
+    char level[2][2*FFT_WINDOW+1][DECLICK_MAX_FFT] ;
+#ifdef HAVE_FFTW3
+    FFTW(plan) pLeft, pRight ;
+#else /* HAVE_FFTW3 */
+    rfftw_plan p ;
+#endif /* HAVE_FFTW3 */
+
+    fftw_real data[2][2*FFT_WINDOW+1] ;
+    fftw_real out[2*DECLICK_MAX_FFT] ;
+    fftw_real window[2*DECLICK_MAX_FFT] ;
+    fftw_real power_spectrum[2*DECLICK_MAX_FFT] ;
+
+    int in_click ;
+    int window_size  = FFT_WINDOW * MIN((p->rate/44100.0),2) ;
+    int window_overlap  = WINDOW_OVERLAP * MIN((p->rate/44100.0),2) ;
+    int window_step ;
+    int channel ;
+    int done = 0 ;
+
+    if(last_sample > p->n_samples-20) last_sample = p->n_samples-20 ;
+    if(first_sample < 20) first_sample = 20 ;
+    if(first_sample >= last_sample) {
+	return "Region to small to declick." ;
+    }
+
+    start_timer();
+    clicks->n_clicks = 0 ;
+
+    n_repaired[0] = n_repaired[1] = 0 ;
+    n_not_repaired[0] = n_not_repaired[1] = 0 ;
+
+    window_step = 700 ;
+    window_size = 801 ;
+
+    g_print("Declick_fft first_sample:%ld last_sample:%ld window_size:%d\n", first_sample, last_sample, window_size) ;
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+
+
+#ifdef HAVE_FFTW3
+    pLeft = FFTW(plan_r2r_1d)(FFT_SIZE, data[0], out, FFTW_R2HC, FFTW_ESTIMATE);
+    pRight = FFTW(plan_r2r_1d)(FFT_SIZE, data[1], out, FFTW_R2HC, FFTW_ESTIMATE);
+#else /* HAVE_FFTW3 */
+    p = rfftw_create_plan(FFT_SIZE, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
+#endif /* HAVE_FFTW3 */
+
+    for(k = 0 ; k < FFT_SIZE ; k++) {
+	 window[k] = blackman(k,FFT_SIZE) ;
+    }
+
+    for(window_first = first_sample ; !done && window_first < last_sample ; window_first += window_step ) {
+	int nclicks_not_repaired_start = clicks->n_clicks ;
+	int clicks_repaired = 1 ;
+	int loop_flag = 1 ;
+	int min_sample,max_sample ;
+
+	if(window_first + window_size > last_sample) {
+	    window_first = last_sample - window_size ;
+	    done = 1 ;
+	}
+
+	double percentage = (double)(window_first-first_sample)/(double)(last_sample-first_sample) ;
+	update_status_bar(percentage,STATUS_UPDATE_INTERVAL,FALSE) ;
+
+	for(i=0 ; i < window_size ; i += 2) {
+
+	    min_sample = i+window_first-FFT_SIZE/2 ;
+	    min_sample = MAX(0, min_sample) ;
+	    max_sample = min_sample+FFT_SIZE-1 ;
+
+	    if(max_sample > p->n_samples-1)
+		max_sample = p->n_samples-1 ;
+
+	    read_fft_real_wavefile_data(data[0],  data[1], min_sample, max_sample) ;
+
+	    for(k = 0 ; k < FFT_SIZE ; k++) {
+		data[0][k] *= window[k] ;
+		data[1][k] *= window[k] ;
+	    }
+
+	    for (channel = 0; channel < 2; channel++) {
+		double min_p = 1.e30, max_p = -1.e30 ;
+		if(! ((channel+1) & channel_mask) ) continue ;
+#ifdef HAVE_FFTW3
+		if (channel == 0)
+		   FFTW(execute)(pLeft);
+		else
+		   FFTW(execute)(pRight);
+#else /* HAVE_FFTW3 */
+		if (channel == 0)
+		   rfftw_one(p, left, out);
+		else
+		   rfftw_one(p, right, out);
+#endif /* HAVE_FFTW3 */
+		power_spectrum[0] = out[0]*out[0];  /* DC component */
+		for (k = 1; k < (FFT_SIZE+1)/2; ++k)  /* (k < FFT_SIZE/2 rounded up) */
+		    power_spectrum[k] = out[k]*out[k] + out[FFT_SIZE-k]*out[FFT_SIZE-k];
+		if (FFT_SIZE % 2 == 0) /* N is even */
+		    power_spectrum[FFT_SIZE/2] = (out[FFT_SIZE/2]*out[FFT_SIZE/2]);  /* Nyquist freq. */
+
+		for(k = 1 ; k <= FFT_SIZE/2 ; k++) {
+		    int y ;
+		    double p = 10.0*log10(power_spectrum[k]) ;
+
+		    if(p < -127.0) p = -127.0 ;
+		    if(p > 127.0) p = 127.0 ;
+
+		    if(p > max_p) max_p = p ;
+		    if(p < min_p) min_p = p ;
+		    
+		    level[channel][i][k-1] = (char)p ;
+		}
+	    }
+	}
+
+	for(channel = 0 ; channel < 2 ; channel++) {
+	    double mean_level[FFT_SIZE] ;
+	    double offset[FFT_WINDOW*2+1] ;
+	    double hgt_sum ;
+	    double mean, stderr, var, cv, stddev ;
+	    long click_start, click_end ;
+
+	    if(! ((channel+1) & channel_mask) ) continue ;
+
+	    for(k = 0 ; k < FFT_SIZE/2 ; k++) {
+		mean_level[k] = 0.0 ;
+		for(i = 0 ; i < window_size ; i += 2)
+		    mean_level[k] += level[channel][i][k] ;
+		mean_level[k] /= (double)window_size ;
+	    }
+
+	    for(i = 0 ; i < window_size ; i += 2) {
+		offset[i] = 0.0 ;
+		for(k = 0 ; k < FFT_SIZE/2 ; k++) {
+		    offset[i] += level[channel][i][k] - mean_level[k] ;
+		}
+
+		offset[i] /= (double)FFT_SIZE/2.0 ;
+	    }
+
+	    for(i = 1 ; i < window_size ; i += 2) {
+		offset[i] = (offset[i-1]+offset[i+1])/2.0 ;
+	    }
+
+/*  	    for(i = 0 ; i < window_size ; i++) {  */
+/*  		if(offset[i] < 0.0) offset[i] = 0.0 ;  */
+/*  	    }  */
+
+	    stats(offset, window_size, &mean, &stderr, &var, &cv, &stddev) ;
+
+	    in_click = 0 ;
+
+	    for(i = 0 ; i < window_size ; i++) {
+		//double z = (offset[i]-mean)/stddev ;
+		double z = (offset[i]-mean) ;
+
+		//printf("i:%d z:%lg\n", i, z) ;
+
+		if(z < 0.0) z = 0.0 ;
+
+		if(z > 1.e-30) {
+		    if(!in_click) {
+			in_click = 1 ;
+			click_start = i+window_first ;
+			hgt_sum = z ;
+		    } else {
+			hgt_sum += z ;
+		    }
+		} else {
+		    if(in_click) {
+			int click_end = i-1+window_first;
+			double click_width = (click_end - click_start+1) ;
+			int click_mid = (click_start + click_width/2+0.5) ;
+			int result = DETECT_ONLY ;
+
+			double mean_hgt = hgt_sum / click_width ;
+
+			if(click_width > 8) {
+			    click_start += click_width/4.0+0.5 ;
+			    /* leave the end, more audio artifacts there that need fixed */
+/*  			    click_end -= click_width/4.0+0.5 ;  */
+			    click_width = (click_end - click_start+1) ;
+			}
+
+
+			if(mean_hgt > sensitivity && click_width < 100) {
+
+			    printf("%d %d %lg\n", channel, click_mid, mean_hgt) ;
+
+			    if(repair == TRUE) {
+				result = declick_a_click(p, click_start, click_end, channel+1) ;
+
+				if(result == SINGULAR_MATRIX) {
+				    click_start -= 10 ;
+				    click_end += 10 ;
+				    click_start = MAX(0, click_start) ;
+				    click_end = MIN(p->n_samples, click_start) ;
+				    result = declick_a_click(p, click_start, click_end, channel+1) ;
+				}
+				    
+			    }
+			} else {
+			    result = REPAIR_OOB ;
+			}
+
+			if(result == REPAIR_OOB) {
+			} else if(result != REPAIR_SUCCESS) {
+			    if(clicks->n_clicks < clicks->max_clicks) {
+				clicks->start[clicks->n_clicks] = click_start ;
+				clicks->end[clicks->n_clicks] = click_end ;
+				clicks->channel[clicks->n_clicks] = channel ;
+				clicks->n_clicks++ ;
+			    } else {
+				if( max_exceeded_notice == 0) {
+				    warning("Exceeded 1000 clicks in selection, additional detections not marked") ;
+				    max_exceeded_notice = 1 ;
+				}
+			    }
+			    n_not_repaired[channel]++ ;
+			} else {
+			    n_repaired[channel]++ ;
+			    n_this_pass ++ ;
+			    clicks_repaired = 1 ;
+			}
+		    }
+		    in_click = 0 ;
+		}
+
+	    }
+	}
+    }
+
+    #ifdef HAVE_FFTW3
+	FFTW(destroy_plan)(pLeft);
+	FFTW(destroy_plan)(pRight);
+    #else /* HAVE_FFTW3 */
+	rfftw_destroy_plan(p);
+    #endif /* HAVE_FFTW3 */
+
+
+    {
+
+	sprintf(results_buf, "%d clicks repaired, %d clicks marked, but remain unrepaired",
+	    n_repaired[0] + n_repaired[1],
+	    n_not_repaired[0] + n_not_repaired[1]) ;
+        g_print("%s\n",results_buf);
+        stop_timer("DECLICK");
+
+
+/*  	warning(results_buf) ;  */
+    }
+
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+
+    return results_buf ;
+}
+
+char *do_declick_hpf(struct sound_prefs *p, long first_sample, long last_sample, int channel_mask, double sensitivity, int repair,
+struct click_data *clicks, int iterate_flag)
 {
     static char results_buf[200] ;
     long min_sample, max_sample ;
@@ -492,7 +782,7 @@ struct click_data *clicks, int iterate_flag)
     window_step = window_size - 2*EXTRA_DATA_WING - window_overlap ;
     if (window_step < 1) {window_step = window_size;}
 
-/*      g_print("Declick first_sample:%ld last_sample:%ld window_size:%d\n", first_sample, last_sample, window_size) ;  */
+    //g_print("Declick first_sample:%ld last_sample:%ld window_size:%d\n", first_sample, last_sample, window_size) ;
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
 
     for(window_first = first_sample ; window_first < last_sample ; window_first += window_step ) {
@@ -570,25 +860,30 @@ struct click_data *clicks, int iterate_flag)
 			    if(click_start < 0) click_start = 0 ;
 			    if(click_end > p->n_samples) click_end = p->n_samples ;
 
-			    if(repair == TRUE) {
-/*  				d_print("Repairing %s start:%ld end:%ld\n", channel == 0 ? "left" : "right", click_start, click_end) ;  */
-/*  				push_status_text("Repairing a click") ;  */
-/*  				update_status_bar(percentage,STATUS_UPDATE_INTERVAL,TRUE) ;  */
-				result = declick_a_click(p, click_start, click_end, channel+1) ;
-
-				if(result == SINGULAR_MATRIX) {
-				    click_start -= 10 ;
-				    click_end += 10 ;
-				    click_start = MAX(0, click_start) ;
-				    click_end = MIN(p->n_samples, click_start) ;
+			    if(click_start >= first_sample && click_end <= last_sample) {
+				if(repair == TRUE) {
+				    //g_print("Repairing %s start:%ld end:%ld\n", channel == 0 ? "left" : "right", click_start, click_end) ;
+    /*  				push_status_text("Repairing a click") ;  */
+    /*  				update_status_bar(percentage,STATUS_UPDATE_INTERVAL,TRUE) ;  */
 				    result = declick_a_click(p, click_start, click_end, channel+1) ;
+
+				    if(result == SINGULAR_MATRIX) {
+					click_start -= 10 ;
+					click_end += 10 ;
+					click_start = MAX(0, click_start) ;
+					click_end = MIN(p->n_samples, click_start) ;
+					result = declick_a_click(p, click_start, click_end, channel+1) ;
+				    }
+					
+    /*  				pop_status_text() ;  */
+    /*  				update_status_bar(percentage,STATUS_UPDATE_INTERVAL,TRUE) ;  */
 				}
-				    
-/*  				pop_status_text() ;  */
-/*  				update_status_bar(percentage,STATUS_UPDATE_INTERVAL,TRUE) ;  */
+			    } else {
+				result = REPAIR_OOB ;
 			    }
 
-			    if(result != REPAIR_SUCCESS) {
+			    if(result == REPAIR_OOB) {
+			    } else if(result != REPAIR_SUCCESS) {
 				if(clicks->n_clicks < clicks->max_clicks) {
 				    clicks->start[clicks->n_clicks] = click_start ;
 				    clicks->end[clicks->n_clicks] = click_end ;
