@@ -179,10 +179,10 @@ static fftw_real windowed[DENOISE_MAX_FFT] ;
 static fftw_real out[DENOISE_MAX_FFT] ;
 
 #ifdef HAVE_FFTW3
-static void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real noise_max2[],
+static void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real noise_max2[], fftw_real noise_avg2[],
                       FFTW(plan) *pFor, FFTW(plan) *pBak,
 #else /* HAVE_FFTW3 */
-void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real noise_max2[],
+void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real noise_max2[], fftw_real noise_avg2[],
                       rfftw_plan *pFor, rfftw_plan *pBak,
 #endif /* HAVE_FFTW3 */
 		      struct denoise_prefs *pPrefs, int ch)
@@ -202,9 +202,9 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
 
     for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
 	windowed[k] = sample[k]*window_coef[k] ;
-/*  	windowed[k] = sample[k] ;  */
-/*  	noise2[k] = noise_min2[k] + amount*(noise_max2[k] - noise_min2[k]) ;  */
 	noise2[k] = noise_max2[k] ;
+	noise2[k] = noise_min2[k] + 0.5*(noise_max2[k] - noise_min2[k]) ;
+	noise2[k] = noise_avg2[k] ;
     }
 
 #ifdef HAVE_FFTW3
@@ -402,14 +402,18 @@ void print_denoise(char *header, struct denoise_prefs *pDnprefs)
 void print_denoise(char *header, struct denoise_prefs *pDnprefs) {}
 #endif
 
+void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs,
+		    long noise_start, long noise_end,
+		    fftw_real *left_noise_min, fftw_real *left_noise_max, fftw_real *left_noise_avg,
+		    fftw_real *right_noise_min, fftw_real *right_noise_max, fftw_real *right_noise_avg) ;
+
 int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noise_start, long noise_end,
             long first_sample, long last_sample, int channel_mask) {
-    long i ;
     long current ;
     int k ;
     fftw_real left[DENOISE_MAX_FFT], right[DENOISE_MAX_FFT] ;
-    fftw_real left_noise_max[DENOISE_MAX_FFT], right_noise_max[DENOISE_MAX_FFT] ;
-    fftw_real left_noise_min[DENOISE_MAX_FFT], right_noise_min[DENOISE_MAX_FFT] ;
+    fftw_real left_noise_max[DENOISE_MAX_FFT], right_noise_max[DENOISE_MAX_FFT], left_noise_avg[DENOISE_MAX_FFT] ;
+    fftw_real left_noise_min[DENOISE_MAX_FFT], right_noise_min[DENOISE_MAX_FFT], right_noise_avg[DENOISE_MAX_FFT] ;
     fftw_real tmp[DENOISE_MAX_FFT] ;
     fftw_real left_prev_frame[DENOISE_MAX_FFT] ;
     fftw_real right_prev_frame[DENOISE_MAX_FFT] ;
@@ -455,72 +459,21 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
     for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
 	window_coef[k] = fft_window(k,pDnprefs->FFT_SIZE, pDnprefs->window_type) ;
 	d_print("k:%d wc:%lg\n", k, window_coef[k]) ;
-	left_noise_max[k] = 0.0 ;
-	right_noise_max[k] = 0.0 ;
-	left_noise_min[k] = DBL_MIN ;
-	right_noise_min[k] = DBL_MIN ;
 	left_prev_frame[k] = 0.0 ;
 	left[k] = 0.0 ;
 	right_prev_frame[k] = 0.0 ;
 	right[k] = 0.0 ;
     }
 
+
     audio_normalize(denoise_normalize) ;
 
-    for(i = 0 ; i < pDnprefs->n_noise_samples ; i++) {
-	long first = noise_start + i*(noise_end-noise_start-pDnprefs->FFT_SIZE)/pDnprefs->n_noise_samples ;
-	double max_left = -1.0 ;
-	read_fft_real_wavefile_data(left,  right, first, first+pDnprefs->FFT_SIZE-1) ;
+    get_noise_sample(pPrefs, pDnprefs, noise_start, noise_end,
+		    left_noise_min, left_noise_max, left_noise_avg,
+		    right_noise_min, right_noise_max, right_noise_avg) ;
 
-	for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
-	    left[k] *= window_coef[k] ;
-	    right[k] *= window_coef[k] ;
-	    max_left = MAX(max_left, ABS(left[k])) ;
-	}
+    audio_normalize(denoise_normalize) ;
 
-#ifdef HAVE_FFTW3
-	FFTW(execute)(pForLeft);
-#else /* HAVE_FFTW3 */
-	rfftw_one(pFor, left, tmp);
-#endif /* HAVE_FFTW3 */
-
-	/* convert noise sample to power spectrum */
-	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
-	    if(k < pDnprefs->FFT_SIZE/2) {
-		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-	    } else {
-		/* Nyquist Frequency */
-		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k]) ;
-		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k]) ;
-	    }
-	}
-
-#ifdef HAVE_FFTW3
-	FFTW(execute)(pForRight);
-#else /* HAVE_FFTW3 */
-	rfftw_one(pFor, right, tmp);
-#endif /* HAVE_FFTW3 */
-
-	/* convert noise sample to power spectrum */
-	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
-	    if(k < pDnprefs->FFT_SIZE/2) {
-		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
-	    } else {
-		/* Nyquist Frequency */
-		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k]) ;
-		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k]) ;
-	    }
-	}
-
-    }
-
-    /* average out the power spectrum samples */
-/*      for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {  */
-/*  	left_noise[k] /= (double)pDnprefs->n_noise_samples ;  */
-/*  	right_noise[k] /= (double)pDnprefs->n_noise_samples ;  */
-/*      }  */
 
 /*      if(smoothness <= 4 || window_type == DENOISE_WINDOW_BLACKMAN_HYBRID)  */
 	s_amount = pDnprefs->amount ;
@@ -560,10 +513,10 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 	    }
 
 	    if(channel_mask & 0x01)
-		fft_remove_noise(left, left_noise_min, left_noise_max,
+		fft_remove_noise(left, left_noise_min, left_noise_max, left_noise_avg,
 		                  &pFor, &pBak, pDnprefs,0) ;
 	    if(channel_mask & 0x02)
-		fft_remove_noise(right, right_noise_min, right_noise_max,
+		fft_remove_noise(right, right_noise_min, right_noise_max, right_noise_avg,
 		                  &pFor, &pBak, pDnprefs,1) ;
 
 	    if(pDnprefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
@@ -623,3 +576,175 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 
     return 0 ;
 }
+
+int print_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noise_start, long noise_end)
+{
+    int i, k ;
+    FILE *fp ;
+
+    fftw_real left_noise_max[DENOISE_MAX_FFT], right_noise_max[DENOISE_MAX_FFT], left_noise_avg[DENOISE_MAX_FFT] ;
+    fftw_real left_noise_min[DENOISE_MAX_FFT], right_noise_min[DENOISE_MAX_FFT], right_noise_avg[DENOISE_MAX_FFT] ;
+
+    get_noise_sample(pPrefs, pDnprefs, noise_start, noise_end,
+		    left_noise_min, left_noise_max, left_noise_avg,
+		    right_noise_min, right_noise_max, right_noise_avg) ;
+
+    fp = fopen("noise.dat", "w") ;
+
+    fprintf(stderr, "FFT_SIZE in print_noise_sample is %d\n", pDnprefs->FFT_SIZE) ;
+
+    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
+	fprintf(fp, "%10lg %lg %lg\n", freq, left_noise_avg[k], right_noise_avg[k]) ;
+    }
+
+    fclose(fp) ;
+}
+
+void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noise_start, long noise_end,
+		    fftw_real *left_noise_min, fftw_real *left_noise_max, fftw_real *left_noise_avg,
+		    fftw_real *right_noise_min, fftw_real *right_noise_max, fftw_real *right_noise_avg)
+{
+    int i, k ;
+#ifdef HAVE_FFTW3
+    FFTW(plan) pForLeft, pForRight ;
+#else /* HAVE_FFTW3 */
+    rfftw_plan pFor, pBak ;
+#endif /* HAVE_FFTW3 */
+
+    fftw_real left[DENOISE_MAX_FFT], right[DENOISE_MAX_FFT] ;
+    fftw_real tmp[DENOISE_MAX_FFT] ;
+
+#ifdef HAVE_FFTW3
+    pForLeft =
+        FFTW(plan_r2r_1d)(pDnprefs->FFT_SIZE, left, tmp, FFTW_R2HC, FFTW_ESTIMATE);
+    pForRight =
+        FFTW(plan_r2r_1d)(pDnprefs->FFT_SIZE, right, tmp, FFTW_R2HC, FFTW_ESTIMATE);
+#else /* HAVE_FFTW3 */
+    pFor = rfftw_create_plan(pDnprefs->FFT_SIZE, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
+    pBak = rfftw_create_plan(pDnprefs->FFT_SIZE, FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE);
+#endif /* HAVE_FFTW3 */
+
+    for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
+	window_coef[k] = fft_window(k,pDnprefs->FFT_SIZE, pDnprefs->window_type) ;
+	left_noise_max[k] = 0.0 ;
+	right_noise_max[k] = 0.0 ;
+	left_noise_avg[k] = 0.0 ;
+	right_noise_avg[k] = 0.0 ;
+	left_noise_min[k] = DBL_MAX ;
+	right_noise_min[k] = DBL_MAX ;
+    }
+
+    audio_normalize(denoise_normalize) ;
+
+    for(i = 0 ; i < pDnprefs->n_noise_samples ; i++) {
+	long first = noise_start + i*(noise_end-noise_start-pDnprefs->FFT_SIZE)/pDnprefs->n_noise_samples ;
+
+	read_fft_real_wavefile_data(left,  right, first, first+pDnprefs->FFT_SIZE-1) ;
+
+	for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
+	    left[k] *= window_coef[k] ;
+	    right[k] *= window_coef[k] ;
+	}
+
+#ifdef HAVE_FFTW3
+	FFTW(execute)(pForLeft);
+#else /* HAVE_FFTW3 */
+	rfftw_one(pFor, left, tmp);
+#endif /* HAVE_FFTW3 */
+
+	/* convert noise sample to power spectrum */
+	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	    if(k < pDnprefs->FFT_SIZE/2) {
+		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
+		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
+		left_noise_avg[k] += tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+	    } else {
+		/* Nyquist Frequency */
+		left_noise_min[k] = MIN(left_noise_min[k], tmp[k] * tmp[k]) ;
+		left_noise_max[k] = MAX(left_noise_max[k], tmp[k] * tmp[k]) ;
+		left_noise_avg[k] += tmp[k] * tmp[k] ;
+	    }
+	}
+
+#ifdef HAVE_FFTW3
+	FFTW(execute)(pForRight);
+#else /* HAVE_FFTW3 */
+	rfftw_one(pFor, right, tmp);
+#endif /* HAVE_FFTW3 */
+
+	/* convert noise sample to power spectrum */
+	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	    if(k < pDnprefs->FFT_SIZE/2) {
+		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
+		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k]) ;
+		right_noise_avg[k] += tmp[k] * tmp[k] + tmp[pDnprefs->FFT_SIZE-k]*tmp[pDnprefs->FFT_SIZE-k] ;
+	    } else {
+		/* Nyquist Frequency */
+		right_noise_min[k] = MIN(right_noise_min[k], tmp[k] * tmp[k]) ;
+		right_noise_max[k] = MAX(right_noise_max[k], tmp[k] * tmp[k]) ;
+		right_noise_avg[k] += tmp[k] * tmp[k] ;
+	    }
+	}
+
+    }
+
+
+    /* average out the power spectrum samples */
+    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	left_noise_avg[k] /= (double)pDnprefs->n_noise_samples ;
+	right_noise_avg[k] /= (double)pDnprefs->n_noise_samples ;
+    }
+
+    if(pDnprefs->freq_filter) {
+	if(pDnprefs->estimate_power_floor) {
+	    double half_freq_w = (pDnprefs->max_sample_freq - pDnprefs->min_sample_freq)/2.0 ;
+	    double sum_left = 0.0 ;
+	    double n_left = 0.0 ;
+	    double sum_right = 0.0 ;
+	    double n_right = 0.0 ;
+	    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+		double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
+		if(freq < pDnprefs->min_sample_freq && freq > pDnprefs->min_sample_freq-half_freq_w) {
+		    sum_left += left_noise_avg[k] ;
+		    n_left += 1.0 ;
+		    sum_right += right_noise_avg[k] ;
+		    n_right += 1.0 ;
+		}
+		if(freq > pDnprefs->max_sample_freq && freq < pDnprefs->max_sample_freq+half_freq_w) {
+		    sum_left += left_noise_avg[k] ;
+		    n_left += 1.0 ;
+		    sum_right += right_noise_avg[k] ;
+		    n_right += 1.0 ;
+		}
+	    }
+	    if(n_left > 1.e-30) sum_left /= n_left ;
+	    if(n_right > 1.e-30) sum_right /= n_left ;
+	    for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+		double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
+		if(freq < pDnprefs->min_sample_freq || freq > pDnprefs->max_sample_freq) {
+		    left_noise_avg[k] -= sum_left ;
+		    right_noise_avg[k] -= sum_right ;
+		}
+	    }
+
+	}
+	for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
+	    double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
+	    if(freq < pDnprefs->min_sample_freq || freq > pDnprefs->max_sample_freq) {
+		left_noise_avg[k] = 0.0 ;
+		right_noise_avg[k] = 0.0 ;
+	    }
+	}
+    }
+
+#ifdef HAVE_FFTW3
+    FFTW(destroy_plan)(pForLeft);
+    FFTW(destroy_plan)(pForRight);
+#else /* HAVE_FFTW3 */
+    rfftw_destroy_plan(pFor);
+    rfftw_destroy_plan(pBak);
+#endif /* HAVE_FFTW3 */
+
+    audio_normalize(1) ;
+}
