@@ -23,8 +23,9 @@
 #include <gnome.h>
 #include "gtkledbar.h"
 #include "gwc.h"
+#include "stat.h"
 
-#define FFT_MAX 8192
+#define FFT_MAX 32768
 
 void print_spectral(char *str, fftw_real tmp_l[], long FFT_SIZE)
 {
@@ -328,7 +329,7 @@ int dethunk_new(struct sound_prefs *pPrefs,
     return 0 ;
 }
 
-int dethunk(struct sound_prefs *pPrefs,
+int dethunk_current(struct sound_prefs *pPrefs,
             long first_sample, long last_sample, int channel_mask)
 {
     long i ;
@@ -380,6 +381,8 @@ int dethunk(struct sound_prefs *pPrefs,
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
 
     g_print("FFTSIZE:%d\n", FFT_SIZE) ;
+    g_print("first_sample:%ld\n", first_sample) ;
+    g_print("last_sample:%ld\n", last_sample) ;
 
 #ifdef HAVE_FFTW3
     pPreLeft   = FFTW(plan_r2r_1d)(FFT_SIZE, left, tmp_l, FFTW_R2HC, FFTW_R2HC);
@@ -415,9 +418,9 @@ int dethunk(struct sound_prefs *pPrefs,
 	for(k = 1 ; k <= FFT_SIZE/2 ; k++) {
 	    if(k < FFT_SIZE/2) {
 		pre_left_amp[k] = tmp_l[k] * tmp_l[k] + tmp_l[FFT_SIZE-k]*tmp_l[FFT_SIZE-k] ;
-		pre_left_phase[k] = atan2(tmp_l[FFT_SIZE-k],tmp_l[k]);
+		pre_left_phase[k] =  atan2(tmp_l[FFT_SIZE-k],tmp_l[k]);
 		pre_right_amp[k] = tmp_r[k] * tmp_r[k] + tmp_r[FFT_SIZE-k]*tmp_r[FFT_SIZE-k] ;
-		pre_right_phase[k] = atan2(tmp_r[FFT_SIZE-k],tmp_r[k]);
+		pre_right_phase[k] =  atan2(tmp_r[FFT_SIZE-k],tmp_r[k]);
 	    } else {
 		/* Nyquist Frequency */
 		pre_left_amp[k] = tmp_l[k] * tmp_l[k] ;
@@ -474,6 +477,11 @@ int dethunk(struct sound_prefs *pPrefs,
 
     }
 
+/*      for(k = 1 ; k <= FFT_SIZE/2 ; k++) {  */
+/*  	printf("pre  k:%d a:%lg p:%lg\n", k, pre_left_amp[k], pre_left_phase[k]) ;  */
+/*  	printf("post k:%d a:%lg p:%lg\n", k, post_left_amp[k], post_left_phase[k]) ;  */
+/*      }  */
+
     read_fft_real_wavefile_data(left,  right, first_sample, first_sample+FFT_SIZE-1) ;
 
     for(i = 0 ; i < FFT_SIZE ; i++) {
@@ -503,6 +511,9 @@ int dethunk(struct sound_prefs *pPrefs,
 	    }
 	}
     }
+
+    g_print("write first_sample:%ld\n", first_sample) ;
+    g_print("write last_sample:%ld\n", last_sample) ;
 	    
     write_fft_real_wavefile_data(left, right, first_sample, last_sample) ;
 
@@ -521,3 +532,146 @@ int dethunk(struct sound_prefs *pPrefs,
 
     return 0 ;
 }
+
+#include "ar.h"
+#include "ar.c"
+
+#define ORDER 2048
+int forward_extrapolate(fftw_real data[], int firstbad, int lastbad, int siglen)
+{
+    int n_bad = lastbad - firstbad + 1 ;
+    int autolen = 60 ;
+    int i, j ;
+    double auto_coefs[ORDER+1] ;
+
+
+/*
+int AutoRegression(
+   double   *inputseries,
+   int      length,
+   int      degree,
+   double   *coefficients,
+   int      method)
+*/
+
+    autolen = (siglen-n_bad)/4 ;
+    //autolen *= 2 ;
+
+    if(autolen < 0) {
+	d_print("Autolen < 0!\n") ;
+	return REPAIR_FAILURE;
+    }
+
+    if(autolen > ORDER) autolen = ORDER ;
+    g_print("siglen:%d n_bad:%d Autolen:%d\n",siglen,n_bad,autolen) ;
+
+    AutoRegression(data,firstbad,autolen,auto_coefs,0) ;
+
+    for(i = firstbad ; i < lastbad ; i++) {
+	data[i] = 0 ;
+	for(j = 0 ; j < autolen ; j++)
+	    data[i] += data[i - j - 1]*auto_coefs[j] ;
+    }
+}
+
+int reverse_extrapolate(fftw_real data[], int firstbad, int lastbad, int siglen)
+{
+    int n_bad = lastbad - firstbad + 1 ;
+    int autolen = 60 ;
+    int i ;
+    fftw_real *x = calloc(siglen, sizeof(fftw_real)) ;
+
+    for(i = 0 ; i < siglen ; i++)
+	x[siglen-i-1] = data[i] ;
+
+    forward_extrapolate(x, siglen-lastbad-1, siglen-firstbad-1, siglen) ;
+
+    for(i = 0 ; i < siglen ; i++)
+	data[i] = x[siglen-i-1] ;
+
+    free(x) ;
+}
+
+void estimate_region(fftw_real data[], int firstbad, int lastbad, int siglen)
+{
+    int i ;
+    fftw_real *data_r = calloc(siglen, sizeof(fftw_real)) ;
+    int n_samples = lastbad - firstbad + 1 ;
+
+    if(n_samples < 1) return ;
+
+    if(n_samples == 1) {
+	data[firstbad] = (data[firstbad-1]+data[firstbad+1])/2.0 ;
+	return ;
+    }
+
+
+    for(i = 0 ; i < siglen ; i++)
+	data_r[siglen-i-1] = data[i] ;
+
+    forward_extrapolate(data, firstbad, lastbad, siglen) ;
+
+    forward_extrapolate(data_r, siglen-lastbad-1, siglen-firstbad-1, siglen) ;
+
+    for(i = firstbad ; i <= lastbad ; i++) {
+	double d = i - firstbad ;
+	double w_r = d / (double)(n_samples-1) ;
+	double w_f = 1.0 - w_r ;
+	data[i] = w_f*data[i] + w_r*data_r[siglen - i - 1] ;
+    }
+
+    free(data_r) ;
+}
+
+
+int dethunk(struct sound_prefs *pPrefs,
+            long first_sample, long last_sample, int channel_mask)
+{
+    long i ;
+    long n_samples = last_sample - first_sample + 1 ;
+    int cancel, k ;
+    fftw_real *left, *right ;
+    int FFT_SIZE = MIN(ORDER*2,(last_sample-first_sample+1)*4) ;
+    int siglen = last_sample-first_sample+1+2*FFT_SIZE ;
+    extern struct view audio_view ;
+
+    left = calloc(siglen, sizeof(fftw_real)) ;
+    right = calloc(siglen, sizeof(fftw_real)) ;
+
+    push_status_text("Saving undo information") ;
+    start_save_undo("Undo dethunk", &audio_view) ;
+    cancel = save_undo_data( first_sample, last_sample, pPrefs, TRUE) ;
+    close_undo() ;
+    pop_status_text() ;
+
+    n_samples = last_sample - first_sample + 1 ;
+
+    push_status_text("Dethunking audio") ;
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+
+    g_print("first_sample:%ld\n", first_sample) ;
+    g_print("last_sample:%ld\n", last_sample) ;
+
+    read_fft_real_wavefile_data(left,  right, first_sample-FFT_SIZE, last_sample+FFT_SIZE) ;
+
+    if(channel_mask & 0x01) {
+	estimate_region(left, FFT_SIZE, FFT_SIZE+n_samples-1, last_sample-first_sample+1+2*FFT_SIZE) ;
+    }
+
+    if(channel_mask & 0x02) {
+	estimate_region(right, FFT_SIZE, FFT_SIZE+n_samples-1, last_sample-first_sample+1+2*FFT_SIZE) ;
+    }
+
+    write_fft_real_wavefile_data(left,  right, first_sample-FFT_SIZE, last_sample+FFT_SIZE) ;
+
+
+    update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
+
+    pop_status_text() ;
+    free(left) ;
+    free(right) ;
+
+    main_redraw(FALSE, TRUE) ;
+
+    return 0 ;
+}
