@@ -330,6 +330,8 @@ int audio_area_motion_event(GtkWidget *c, GdkEventMotion *event)
 	if(selecting_region == TRUE) {
 	    long marker_pix ;
 	    int i ;
+	    int min_marker_dist_to_first = 10 ;
+	    int min_marker_dist_to_last = 10 ;
 
 	    last_pick_x = x ;
 	    region_select_min_x = MIN(first_pick_x, last_pick_x) ;
@@ -347,17 +349,37 @@ int audio_area_motion_event(GtkWidget *c, GdkEventMotion *event)
 
 		marker_pix = sample_to_pixel(&audio_view, markers[i]) ;
 
-		if( ABS(region_select_min_x-marker_pix) < 10) {
-		    region_select_min_x = marker_pix ;
+		if( ABS(region_select_min_x-marker_pix) < min_marker_dist_to_first) {
+		    min_marker_dist_to_first = ABS(region_select_min_x-marker_pix) ;
+		    //region_select_min_x = marker_pix ;
 		    audio_view.selected_first_sample = markers[i] ;
 		}
 
-		if( ABS(region_select_max_x-marker_pix) < 10) {
-		    region_select_max_x = marker_pix ;
+		if( ABS(region_select_max_x-marker_pix) < min_marker_dist_to_last) {
+		    min_marker_dist_to_last = ABS(region_select_max_x-marker_pix) ;
+		    //region_select_max_x = marker_pix ;
 		    audio_view.selected_last_sample = markers[i] ;
 		}
 	    }
 
+	    for(i = 0 ; i < num_song_markers ; i++) {
+		if(song_markers[i] < 0) continue ;
+
+		marker_pix = sample_to_pixel(&audio_view, song_markers[i]) ;
+
+		if( ABS(region_select_min_x-marker_pix) < min_marker_dist_to_first) {
+		    min_marker_dist_to_first = ABS(region_select_min_x-marker_pix) ;
+		    //region_select_min_x = marker_pix ;
+		    audio_view.selected_first_sample = song_markers[i] ;
+		}
+
+		if( ABS(region_select_max_x-marker_pix) < min_marker_dist_to_last) {
+		    min_marker_dist_to_last = ABS(region_select_max_x-marker_pix) ;
+		    //region_select_max_x = marker_pix ;
+		    audio_view.selected_last_sample = song_markers[i] ;
+		}
+	    }
+
 	    audio_view.selection_region = TRUE ;
 
 	    display_times() ;
@@ -437,12 +459,10 @@ void draw_sonogram(struct view *v, struct sound_prefs *pPrefs, GtkWidget *da, do
     double spectrum_scale;
     int index_20Hz;
 
-#define TEST_DRAW_CLICK_FINDER_NOT
-#ifdef TEST_DRAW_CLICK_FINDER
-    double left_hpf[1600], right_hpf[1600] ;
+#define MAXSW 2000
+#define MAXSH 400
+    unsigned char level[2][MAXSW][MAXSH] ;
     int in_click ;
-#endif
-#
 
     push_status_text("Building sonogram") ;
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
@@ -467,6 +487,7 @@ void draw_sonogram(struct view *v, struct sound_prefs *pPrefs, GtkWidget *da, do
     for(FFT_SIZE = 64 ; FFT_SIZE < FFT_MAX ; FFT_SIZE *= 2)
 	if(FFT_SIZE > samples_per_pixel*2)
 	    break ;
+
     m = FFT_SIZE/4 ;
 
     index_20Hz = MAX(1,(20 * FFT_SIZE + pPrefs->rate / 2) / pPrefs->rate);
@@ -534,11 +555,6 @@ ly2_tbl[0] = 99999999;
 	mid_sample = (max_sample-min_sample+1)/2 ;
 	read_fft_real_wavefile_data(left,  right, min_sample, max_sample) ;
 
-#ifdef TEST_DRAW_CLICK_FINDER
-	left_hpf[x] = high_pass_filter(left+mid_sample-8, 17) ;
-	right_hpf[x] = high_pass_filter(right+mid_sample-8, 17) ;
-#endif
-
 	for(k = 0 ; k < FFT_SIZE ; k++) {
 	    left[k] *= window[k] ;
 	    right[k] *= window[k] ;
@@ -629,14 +645,6 @@ ly2_tbl[0] = 99999999;
 		color = psk_to_color(merged_power_spectrum[k], spectrum_scale, 255) ;
 		if(color > 255) color=255 ;
 		if(color < 0) color=0 ; 
-#if 0
-                if (channel == 0) {
-		   int c = (int) (k*255.0/(spec_stop-spec_start)) ;
-		   if(c > 255) c = 255 ;
-		   if(c < 0) c = 0 ;
-		   gdk_image_put_pixel(image, x, k + ly_offset[channel], sonogram_color[c].pixel);
-		} else
-#endif
 		gdk_image_put_pixel(image, x, k + ly_offset[channel], sonogram_color[color].pixel);
 	    }
 #else
@@ -644,20 +652,10 @@ ly2_tbl[0] = 99999999;
 	    for(k = 1 ; k <= FFT_SIZE/2 ; k++) {
 		int y ;
 		color = psk_to_color(power_spectrum[k]*spectral_amp, 1.0, 255) ;
-		if(color > 255) color=255 ;
 		if(color < 0) color=0 ; 
-#if 0
-                if (channel == 0) {
-		   int c = (int) (k*255.0/(FFT_SIZE/2.0)) ;
-		   if(c > 255) c = 255 ;
-		   if(c < 0) c = 0 ;
-		   gdk_image_put_pixel(image, x, ly2_tbl[k] + ly_offset[channel], sonogram_color[c].pixel);
-		} else
-#endif
+		level[channel][x][k-1] = color ;
+		if(color > 255) color=255 ;
 		{ 
-#if 0
-		    if(x == 1) g_print("sonogram k:%d ly2:%d ly1:%d\n", k, ly2_tbl[k], ly1_tbl[k]) ;
-#endif
 		    for(y = ly2_tbl[k] ; y < ly1_tbl[k] ; y++) {
 			gdk_image_put_pixel(image, x, y + ly_offset[channel], sonogram_color[color].pixel);
 		    }
@@ -668,10 +666,80 @@ ly2_tbl[0] = 99999999;
 #endif
         }
     }
+
     gdk_draw_image(audio_pixmap, da->style->white_gc, image, 0, 0, 0, 0,
        v->canvas_width, v->canvas_height); 
     gdk_image_destroy(image);
 
+    for(channel = 0 ; channel < 2 ; channel++) {
+	double mean_level[MAXSH] ;
+	double offset[MAXSW] ;
+	double hgt_sum ;
+	static double mean, stderr, var, cv, stddev ;
+	long click_start, click_end ;
+
+	for(k = 0 ; k < FFT_SIZE/2 ; k++) {
+	    mean_level[k] = 0.0 ;
+	    for(x = 0 ; x < v->canvas_width ; x++)
+		mean_level[k] += level[channel][x][k] ;
+	    mean_level[k] /= v->canvas_width ;
+	}
+
+	for(x = 0 ; x < v->canvas_width ; x++) {
+	    offset[x] = 0.0 ;
+	    for(k = 0 ; k < FFT_SIZE/2 ; k++) {
+		offset[x] += level[channel][x][k] - mean_level[k] ;
+	    }
+
+	    offset[x] /= (double)FFT_SIZE/2.0 ;
+	    if(offset[x] < 0.0) offset[x] = 0.0 ;
+
+	}
+
+	stats(offset, v->canvas_width, &mean, &stderr, &var, &cv, &stddev) ;
+
+	in_click = 0 ;
+
+	for(x = 0 ; x < v->canvas_width-1 ; x++) {
+	    double z = (offset[x]-mean)/stddev ;
+	    double z1 = (offset[x+1]-mean)/stddev ;
+
+	    if(z < 0.0) z = 0.0 ;
+	    if(z1 < 0.0) z1 = 0.0 ;
+
+	    ly1 = AtoS(-40.0*z,200.0,channel,v->canvas_height) ;
+	    ly2 = AtoS(-40.0*z1,200.0,channel,v->canvas_height) ;
+
+	    if(z > 1.e-30) {
+		if(!in_click) {
+		    in_click = 1 ;
+		    click_start = x ;
+		    hgt_sum = z ;
+		} else {
+		    hgt_sum += z ;
+		}
+	    } else {
+		if(in_click) {
+		    double click_width = ((x-1) - click_start+1) ;
+		    int click_mid = (click_start + click_width/2+0.5) ;
+		    int y1,y2 ;
+
+		    hgt_sum /= click_width ;
+		    //printf("%d %lg\n", click_mid, hgt_sum) ;
+
+		    y1 = AtoS(0.0,200.0,channel,v->canvas_height) ;
+		    y2 = AtoS(-40.0*hgt_sum,200.0,channel,v->canvas_height) ;
+		    draw_a_line(da->style->white_gc, click_mid-1, y1, click_mid-1, y2, green_color, 0) ;
+		    draw_a_line(da->style->white_gc, click_mid, y1, click_mid, y2, green_color, 0) ;
+		    draw_a_line(da->style->white_gc, click_mid+1, y1, click_mid+1, y2, green_color, 0) ;
+		}
+		in_click = 0 ;
+	    }
+
+	    draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, green_color, 0) ;
+	}
+    }
+
 #ifdef HAVE_FFTW3
     FFTW(destroy_plan)(pLeft);
     FFTW(destroy_plan)(pRight);
@@ -699,69 +767,6 @@ ly2_tbl[0] = 99999999;
 
 	ly1 = AtoS(2.5-ABS(1.0),5.0,1,v->canvas_height) ;
 	draw_a_line(da->style->white_gc, 0, ly1, v->canvas_width-1, ly1, red_color, 0) ;
-
-	for(x = v->canvas_width-2 ; x >= 0 ; x--) {
-	    static double mean, stderr, var, cv, stddev ;
-	    ly1 = AtoS(100-left_hpf[x],200.0,0,v->canvas_height) ;
-	    ly2 = AtoS(100-left_hpf[x+1],200.0,0,v->canvas_height) ;
-	    draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, white_color, 0) ;
-
-
-    /*  	if(v->canvas_width-pixel > N) {  */
-    /*  	    if(!in_click) {  */
-    /*  		stats(&right_hpf[x+1], N, &mean, &stderr, &var, &cv, &stddev) ;  */
-    /*  		if(right_hpf[x]/mean > 2.0 && right_hpf[x] > 10.0*stderr+mean) {  */
-    /*  		    in_click = 1 ;  */
-    /*  		}  */
-    /*  	    } else {  */
-    /*  		if(mean/right_hpf[x] < 2.0 && right_hpf[x] < 10.0*stderr+mean) {  */
-    /*  		    in_click = 0 ;  */
-    /*  		}  */
-    /*  	    }  */
-    /*  	}  */
-
-/*              d_print("pixel:%d right_hpf:%g mean:%g stddev:%g mean_right:%g\n", x, right_hpf[x], mean, stddev, mean_right) ;  */
-
-	    if(v->canvas_width-x > N) {
-		int n = N, i ;
-		double dy[N] ;
-		double dy2 = (right_hpf[x] - right_hpf[x+2])*5.0 ;
-		double dy1 = (right_hpf[x-1] - right_hpf[x+1])*5.0 ;
-		double zy1, zy2 ;
-
-		for(i = 0 ; i < N; i++)
-		    dy[i] = (right_hpf[x+i-1] - right_hpf[x+i+1])*5.0 ;
-
-		if(!in_click) stats(dy, N, &mean, &stderr, &var, &cv, &stddev) ;
-
-/*  		if(x > 600-N-50)  */
-/*  		    d_print("pixel:%d z:%g mean:%g stderr:%g\n", x, (dy1-mean)/stddev, mean, stderr) ;  */
-
-		if(in_click == 0 && dy1 > 2.00*stddev+mean && right_hpf[x] > 2.0*mean_right) {
-		    in_click = 1 ;
-		} else if(in_click == 1 && right_hpf[x] < 2.0*mean_right) {
-		    in_click = 0 ;
-		}
-
-		zy1 = ABS(dy1-mean)/stddev ;
-		zy2 = ABS(dy2-mean)/stddev ;
-
-		ly1 = AtoS(5-ABS(zy1),10.0,1,v->canvas_height) ;
-		ly2 = AtoS(5-ABS(zy2),10.0,1,v->canvas_height) ;
-		if(dy1 > 0)
-		    draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, blue_color, 0) ;
-		else
-		    draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, yellow_color, 0) ;
-	    }
-
-	    ly1 = AtoS(100-right_hpf[x],200.0,1,v->canvas_height) ;
-	    ly2 = AtoS(100-right_hpf[x+1],200.0,1,v->canvas_height) ;
-
-	    if(in_click)
-		draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, red_color, 0) ;
-	    else
-		draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, white_color, 0) ;
-	}
     }
 #endif
 
@@ -1049,6 +1054,11 @@ void redraw(struct view *v, struct sound_prefs *p, GtkWidget *da, int cursor_fla
 	    sonogram_color[i].red = 65535*pow(p_red, gamma_fact) ;
 	    sonogram_color[i].green = 65535*pow(p_green, gamma_fact) ;
 	    sonogram_color[i].blue = 65535*pow(p_blue, gamma_fact) ;
+#ifdef INVERT_SONOGRAM
+	    sonogram_color[i].red = 65535 - sonogram_color[i].red ;
+	    sonogram_color[i].green = 65535 - sonogram_color[i].green ;
+	    sonogram_color[i].blue = 65535 - sonogram_color[i].blue ;
+#endif
 
 	    gdk_colormap_alloc_color(my_map, &sonogram_color[i], FALSE, TRUE) ;
 	}
@@ -1118,13 +1128,15 @@ void redraw(struct view *v, struct sound_prefs *p, GtkWidget *da, int cursor_fla
 
     { 
 	for (i = 0; i < num_song_markers; i++) {
-	    gint x = sample_to_pixel(v, song_markers[i]) ;
-	    draw_a_line(MyGC, x, 0, x, v->canvas_height-1, orange_color, 0) ;
+	    if(song_markers[i] >= v->first_sample && song_markers[i] <= v->last_sample) {
+		gint x = sample_to_pixel(v, song_markers[i]) ;
+		draw_a_line(MyGC, x, 0, x, v->canvas_height-1, orange_color, 0) ;
+	    }
 	}
     }
 
     for(i = 0 ; i < n_markers ; i++) {
-	if(markers[i] > 0 && markers[i] < v->n_samples) {
+	if(markers[i] >= v->first_sample && markers[i] <= v->last_sample) {
 	    int x = sample_to_pixel(v, markers[i]) ;
 	    draw_a_line(MyGC, x, 0, x, v->canvas_height-1, blue_color, 0) ;
 	}
