@@ -45,11 +45,13 @@
 #include <sndfile.h>
 
 #include "icons/amplify.xpm"
+#include "icons/pinknoise.xpm"
 #include "icons/declick.xpm"
 #include "icons/declick_w.xpm"
 #include "icons/declick_m.xpm"
 #include "icons/decrackle.xpm"
 #include "icons/estimate.xpm"
+#include "icons/filter.xpm"
 #include "icons/noise_sample.xpm"
 #include "icons/remove_noise.xpm"
 #include "icons/cut.xpm"
@@ -83,6 +85,7 @@ GtkWidget *detect_only_widget;
 GtkWidget *l_file_time;
 GtkWidget *l_file_samples;
 GtkWidget *l_first_time;
+GtkWidget *l_selected_time;
 GtkWidget *l_last_time;
 GtkWidget *l_samples;
 
@@ -92,9 +95,10 @@ struct encoding_prefs encoding_prefs;
 struct view audio_view;
 struct click_data click_data;
 int audio_playback = FALSE;
+int audio_is_looping = FALSE;
 int cursor_playback = FALSE;
 long cursor_samples_per_playback_block;
-gint playback_timer;
+gint playback_timer = -1 ;
 gint cursor_timer;
 gint spectral_view_flag = FALSE;
 gint repair_clicks = 1;
@@ -102,6 +106,9 @@ double view_scale = 1.0;
 double declick_sensitivity = 0.75;
 double weak_declick_sensitivity = 1.00;
 double strong_declick_sensitivity = 0.75;
+double weak_fft_declick_sensitivity = 3.0 ;
+double strong_fft_declick_sensitivity = 5.0 ;
+int declick_detector_type = FFT_DETECT ;
 double stop_key_highlight_interval;
 double song_key_highlight_interval;
 double song_mark_silence;
@@ -141,6 +148,7 @@ GtkWidget *file_selector;
 gchar *selected_filename;
 gint file_is_open = FALSE;
 gint file_processing = FALSE;
+int stop_playback_force = 1 ;
 
 void d_print(char *fmt, ...)
 {
@@ -243,6 +251,9 @@ void display_times(void)
     gtk_label_set_text(GTK_LABEL(l_last_time),
 		       sample_to_time_text(last, prefs.rate, "Last ",
 					   buf));
+    gtk_label_set_text(GTK_LABEL(l_selected_time),
+		       sample_to_time_text(last-first-1, prefs.rate, "Selected ",
+					   buf));
     sprintf(buf, "Samples: %ld", last - first + 1);
     gtk_label_set_text(GTK_LABEL(l_samples), buf);
     sprintf(buf, "Track samples: %ld", audio_view.n_samples);
@@ -273,7 +284,7 @@ void set_scroll_bar(long n, long first, long last)
     a->value = df;
     a->page_size = ps;
     a->step_increment = ps / 8.0;
-    a->page_increment = ps;
+    a->page_increment = ps * 0.95;
 
 /*      scroll_pos = gtk_adjustment_new(1.0, 0.0, 100.0, 10.0, 20.0, 20.0) ;  */
 /*      a->lower = 0 ;  */
@@ -312,7 +323,7 @@ void get_region_of_interest(long *first, long *last, struct view *v)
 
 void load_preferences(void)
 {
-    gnome_config_push_prefix(CONFIGDIR);
+    gnome_config_push_prefix(APPNAME"/config/");
     strcpy(pathname, gnome_config_get_string("pathname=./"));
     strcpy(last_filename, gnome_config_get_string("last_filename=./"));
     prefs.rate = gnome_config_get_int("rate=44100");
@@ -324,11 +335,12 @@ void load_preferences(void)
     num_song_markers = 0;
     audio_view.channel_selection_mask =
 	gnome_config_get_int("channel_selection_mask=0");
-    weak_declick_sensitivity =
-	gnome_config_get_float("weak_declick_sensitivity=1.0");
-    strong_declick_sensitivity =
-	gnome_config_get_float("strong_declick_sensitivity=0.75");
+    weak_declick_sensitivity = gnome_config_get_float("weak_declick_sensitivity=1.0");
+    strong_declick_sensitivity = gnome_config_get_float("strong_declick_sensitivity=0.75");
     declick_iterate_flag = gnome_config_get_int("declick_iterate=0");
+    weak_fft_declick_sensitivity = gnome_config_get_float("weak_fft_declick_sensitivity=3.0");
+    strong_fft_declick_sensitivity = gnome_config_get_float("strong_fft_declick_sensitivity=5.0");
+    declick_detector_type = gnome_config_get_int("declick_detector_type=0");
     decrackle_level = gnome_config_get_float("decrackle_level=0.2");
     decrackle_window = gnome_config_get_int("decrackle_window=2000");
     decrackle_average = gnome_config_get_int("decrackle_average=3");
@@ -351,7 +363,7 @@ void load_preferences(void)
 
 void save_preferences(void)
 {
-    gnome_config_push_prefix(CONFIGDIR);
+    gnome_config_push_prefix(APPNAME"/config/");
     gnome_config_set_string("pathname", pathname);
     gnome_config_set_string("last_filename", last_filename);
     gnome_config_set_int("rate", prefs.rate);
@@ -361,11 +373,12 @@ void save_preferences(void)
     gnome_config_set_int("last_sample_viewed", audio_view.last_sample);
     gnome_config_set_int("channel_selection_mask",
 			 audio_view.channel_selection_mask);
-    gnome_config_set_float("weak_declick_sensitivity",
-			   weak_declick_sensitivity);
-    gnome_config_set_float("strong_declick_sensitivity",
-			   strong_declick_sensitivity);
+    gnome_config_set_float("weak_declick_sensitivity", weak_declick_sensitivity);
+    gnome_config_set_float("strong_declick_sensitivity", strong_declick_sensitivity);
     gnome_config_set_int("declick_iterate", declick_iterate_flag);
+    gnome_config_set_float("weak_fft_declick_sensitivity", weak_fft_declick_sensitivity);
+    gnome_config_set_float("strong_fft_declick_sensitivity", strong_fft_declick_sensitivity);
+    gnome_config_set_int("declick_detector_type", declick_detector_type);
     gnome_config_set_float("decrackle_level", decrackle_level);
     gnome_config_set_int("decrackle_window", decrackle_window);
     gnome_config_set_int("decrackle_average", decrackle_average);
@@ -569,8 +582,14 @@ void declick(GtkWidget * widget, gpointer data)
 {
     if ((file_processing == FALSE) && (file_is_open == TRUE)
 	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
+
 	file_processing = TRUE;
-	declick_with_sensitivity(strong_declick_sensitivity);
+
+	if(declick_detector_type == HPF_DETECT)
+	    declick_with_sensitivity(strong_declick_sensitivity);
+	else
+	    declick_with_sensitivity(strong_fft_declick_sensitivity);
+
 	file_processing = FALSE;
     }
 }
@@ -579,8 +598,14 @@ void declick_weak(GtkWidget * widget, gpointer data)
 {
     if ((file_processing == FALSE) && (file_is_open == TRUE)
 	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
+
 	file_processing = TRUE;
-	declick_with_sensitivity(weak_declick_sensitivity);
+
+	if(declick_detector_type == HPF_DETECT)
+	    declick_with_sensitivity(weak_declick_sensitivity);
+	else
+	    declick_with_sensitivity(weak_fft_declick_sensitivity);
+
 	file_processing = FALSE;
     }
 }
@@ -679,6 +704,8 @@ void noise_sample(GtkWidget * widget, gpointer data)
 	get_region_of_interest(&denoise_data.noise_start,
 			       &denoise_data.noise_end, &audio_view);
 	denoise_data.ready = TRUE;
+	load_denoise_preferences() ;
+	print_noise_sample(&prefs, &denoise_prefs, denoise_data.noise_start, denoise_data.noise_end) ;
 	file_processing = FALSE;
     }
 }
@@ -704,6 +731,7 @@ void remove_noise(GtkWidget * widget, gpointer data)
 		    warning
 			("FFT_SIZE must be <= # samples in noise sample!");
 		    main_redraw(FALSE, TRUE);
+		    file_processing = FALSE ;
 		    return;
 		}
 
@@ -858,10 +886,13 @@ void silence_callback(GtkWidget * widget, gpointer data)
     if ((file_processing == FALSE) && (file_is_open == TRUE)
 	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
         if (is_region_selected()) {
-            file_processing = TRUE;
-            audioedit_insert_silence(&audio_view);
-            main_redraw(FALSE, TRUE);
-            file_processing = FALSE;
+
+	    if(!yesno("Insert silence can take a long time, continue?")) {
+		file_processing = TRUE;
+		audioedit_insert_silence(&audio_view);
+		main_redraw(FALSE, TRUE);
+		file_processing = FALSE;
+	    }
         }
     }
 }
@@ -898,11 +929,16 @@ long prev_cursor_millisec = -1;
 * in this example. More on callbacks below. */
 void stop_all_playback_functions(GtkWidget * widget, gpointer data)
 {
-    if (audio_playback == TRUE)
+    if (playback_timer != -1) {
 	gtk_timeout_remove(playback_timer);
+	playback_timer = -1 ;
+    }
+
     if (cursor_playback == TRUE)
 	gtk_timeout_remove(cursor_timer);
-    stop_playback();
+
+    stop_playback(stop_playback_force);
+
     cursor_playback = FALSE;
     audio_playback = FALSE;
     led_bar_light_percent(dial[0], 0.0);
@@ -944,10 +980,11 @@ gint play_a_block(gpointer data)
     bytes_left =
 	set_playback_cursor_position(&audio_view, prev_cursor_millisec);
 
-    if (bytes_left < 10) {	/* the  "10" is to allow some error in the audio driver */
+    if (bytes_left < 10 && !audio_is_looping) {	/* the  "10" is to allow some error in the audio driver */
 /*  	g_print("play_a_block is stopping the playback_timer.\n") ;  */
-	gtk_timeout_remove(playback_timer);
-	audio_playback = FALSE;
+	stop_playback_force = 0 ;
+	stop_all_playback_functions(NULL, NULL) ;
+	stop_playback_force = 1 ;
     }
 
 
@@ -991,7 +1028,9 @@ gint update_cursor(gpointer data)
 /*  	    g_print("update_cursor is stopping cursor_timer\n") ;  */
 	    cursor_playback = FALSE;
 	    gtk_timeout_remove(cursor_timer);
+	    stop_playback_force = 0 ;
 	    stop_all_playback_functions(NULL, NULL);
+	    stop_playback_force = 1 ;
 	    prev_cursor_millisec = -1;
 /*          this will redraw the hole sonogram view at the
             end of a "full view" playback  ...frank 31.08.03 */
@@ -1010,8 +1049,9 @@ void start(GtkWidget * widget, gpointer data)
     if (file_is_open == TRUE && file_processing == FALSE
 	&& audio_playback == FALSE && cursor_playback == FALSE) {
 	audio_playback = TRUE;
-	playback_samples_per_block =
-	    start_playback(audio_device, &audio_view, &prefs, 0.10, 0.25);
+
+	playback_samples_per_block = start_playback(audio_device, &audio_view, &prefs, 0.10, 0.25);
+
 	if (audio_playback == TRUE) {
 	    millisec_per_block =
 		playback_samples_per_block * 1000 / prefs.rate;
@@ -1036,6 +1076,10 @@ void start(GtkWidget * widget, gpointer data)
 	       device now! */
 	}
     }
+
+    g_print("leaving start with audio_playback=%d\n", audio_playback) ;
+
+    audio_is_looping = FALSE ;
 }
 
 void detect_only_func(GtkWidget * widget, gpointer data)
@@ -1081,6 +1125,70 @@ void amplify(GtkWidget * widget, gpointer data)
     }
 }
 
+/* This is a callback function. The data arguments are ignored
+* in this example. More on callbacks below. */
+void pinknoise_cb(GtkWidget * widget, gpointer data)
+{
+    if ((file_processing == FALSE) && (file_is_open == TRUE)
+	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
+	file_processing = TRUE;
+
+	if (pinknoise_dialog(prefs, &audio_view)) {
+	    long first, last;
+	    int cancel;
+	    get_region_of_interest(&first, &last, &audio_view);
+
+	    push_status_text("Saving undo information");
+	    start_save_undo("Undo pinknoise", &audio_view);
+	    cancel = save_undo_data(first, last, &prefs, TRUE);
+	    close_undo();
+	    pop_status_text();
+
+	    if (cancel != 1) {
+		pinknoise(&prefs, first, last,
+			      audio_view.channel_selection_mask);
+		save_sample_block_data(&prefs);
+		set_status_text("Amplify done.");
+	    }
+	    main_redraw(FALSE, TRUE);
+	}
+
+	file_processing = FALSE;
+    }
+}
+
+/* This is a callback function. The data arguments are ignored
+* in this example. More on callbacks below. */
+void filter_cb(GtkWidget * widget, gpointer data)
+{
+    if ((file_processing == FALSE) && (file_is_open == TRUE)
+	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
+	file_processing = TRUE;
+
+	if (filter_dialog(prefs, &audio_view)) {
+	    long first, last;
+	    int cancel;
+	    get_region_of_interest(&first, &last, &audio_view);
+
+	    push_status_text("Saving undo information");
+	    start_save_undo("Undo filter", &audio_view);
+	    cancel = save_undo_data(first, last, &prefs, TRUE);
+	    close_undo();
+	    pop_status_text();
+
+	    if (cancel != 1) {
+		filter_audio(&prefs, first, last,
+			      audio_view.channel_selection_mask);
+		save_sample_block_data(&prefs);
+		set_status_text("Amplify done.");
+	    }
+	    main_redraw(FALSE, TRUE);
+	}
+
+	file_processing = FALSE;
+    }
+}
+
 void zoom_select(GtkWidget * widget, gpointer data)
 {
     if ((file_processing == FALSE) && (file_is_open == TRUE)
@@ -1304,6 +1412,15 @@ gboolean  key_press_cb(GtkWidget * widget, GdkEventKey * event, gpointer data)
 		stop_all_playback_functions(widget, data);
 	    }
 	    break;
+	case GDK_l:
+	    if (cursor_playback == FALSE) {
+		start(widget, data);
+		audio_is_looping = TRUE ;
+	    } else {
+		audio_is_looping = FALSE ;
+		stop_all_playback_functions(widget, data);
+	    }
+	    break;
 	case GDK_s:
 	    if (audio_playback == TRUE) {
 		set_playback_cursor_position(&audio_view,
@@ -1372,7 +1489,7 @@ gint delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
 
 void cleanup_and_close(void)
 {
-    stop_playback();
+    stop_playback(1);
 
     if (file_is_open)
 	save_sample_block_data(&prefs);
@@ -1392,10 +1509,10 @@ void destroy(GtkWidget * widget, gpointer data)
 
 void about(GtkWidget * widget, gpointer data)
 {
-    const gchar *authors[] = { "Jeffrey J. Welty", "James Tappin", "Ian Leonard", "Bill Jetzer", "Charles Morgon", "Frank Freudenberg", NULL };
+    const gchar *authors[] = { "Jeffrey J. Welty", "James Tappin", "Ian Leonard", "Bill Jetzer", "Charles Morgon", "Frank Freudenberg", "Thiemo Gehrke", NULL };
     gtk_widget_show(gnome_about_new("Gnome Wave Cleaner",
 				    VERSION,
-				    "Copyright 2001,2002,2003,2004 Redhawk.org",
+				    "Copyright 2001,2002,2003,2004,2005 Redhawk.org",
 				    "An application to aid in denoising (hiss & clicks) of audio files",
 				    authors,
 				    NULL,
@@ -1750,14 +1867,15 @@ GnomeUIInfo file_menu[] = {
 GnomeUIInfo edit_menu[] = {
     GNOMEUIINFO_MENU_UNDO_ITEM(undo_callback, NULL),
 
-#ifdef TRUNCATE_OLD
-    GNOMEUIINFO_ITEM("    Amplify",
-		     "Amplify the current view or selection",
-		     amplify, amplify_xpm),
-#else
+    GNOMEUIINFO_ITEM("    Apply DSP Frequency Filters", "lowpass,highpass,notch or bandpass biquad filtering",
+		     filter_cb, filter_xpm),
+
+    GNOMEUIINFO_ITEM("    Generate Pink Noise", "Replace current view or selection with pink noise",
+		     pinknoise_cb, pinknoise_xpm),
+
     GNOMEUIINFO_ITEM("    Amplify", "Amplify the current view or selection",
 		     amplify, amplify_xpm),
-#endif
+
     GNOMEUIINFO_ITEM("    Declick Strong",
 		     "Remove pops/clicks from current view or selection",
 		     declick, declick_xpm),
@@ -2280,7 +2398,7 @@ int main(int argc, char *argv[])
     {
 	struct  rlimit   rl;
 
-#define  GWC_STACK_LIM (4 * 1024 *1024)
+#define  GWC_STACK_LIM (6 * 1024 *1024)
 
 	if( getrlimit( RLIMIT_STACK, &rl ) == -1 )
 	{
@@ -2293,7 +2411,7 @@ int main(int argc, char *argv[])
 	/*   Only change the size if it's too small.   */
 	if( rl.rlim_cur < GWC_STACK_LIM )
 	{
-	    rl.rlim_cur = 4 * 1024 * 1024;
+	    rl.rlim_cur = GWC_STACK_LIM ;
 
 	    if( setrlimit( RLIMIT_STACK, &rl ) == -1 )
 	    {
@@ -2411,7 +2529,7 @@ int main(int argc, char *argv[])
 			   TRUE, 0);
     }
 
-    scroll_pos = gtk_adjustment_new(1.0, 0.0, 100.0, 10.0, 20.0, 20.0);
+    scroll_pos = gtk_adjustment_new(1.0, 0.0, 100.0, 5.0, 5.0, 0.0);
     hscrollbar = gtk_hscrollbar_new(GTK_ADJUSTMENT(scroll_pos));
     gtk_widget_show(hscrollbar);
 
@@ -2422,7 +2540,7 @@ int main(int argc, char *argv[])
 		       "value_changed", (GtkSignalFunc) scroll_bar_changed,
 		       NULL);
 
-    gtk_box_pack_start(GTK_BOX(main_vbox), hscrollbar, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(main_vbox), hscrollbar, FALSE, TRUE, 0);
 
     {
 	edit_toolbar = gtk_toolbar_new() ;
@@ -2457,6 +2575,7 @@ int main(int argc, char *argv[])
     l_first_time =
 	mk_label_and_pack(GTK_BOX(times_vbox), "First 0:00:000");
     l_last_time = mk_label_and_pack(GTK_BOX(times_vbox), "Last 0:00:000");
+    l_selected_time = mk_label_and_pack(GTK_BOX(times_vbox), "Selected 0:00:000");
     l_samples = mk_label_and_pack(GTK_BOX(times_vbox), "Samples: 0");
 
     for (i = 0; i < 2; i++) {
@@ -2473,7 +2592,7 @@ int main(int argc, char *argv[])
 		       0);
     gtk_box_pack_start(GTK_BOX(bottom_hbox), times_vbox, TRUE, TRUE, 0);
 
-    gtk_box_pack_start(GTK_BOX(main_vbox), bottom_hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(main_vbox), bottom_hbox, FALSE, TRUE, 0);
 
     {
 	detect_only_box = gtk_hbox_new(FALSE, 10);
