@@ -44,7 +44,7 @@
 
 #define GWC_VERSION_MAJOR 0
 #define GWC_VERSION_MINOR 20
-#define VERSION "0.20-07"
+#define VERSION "0.20-10"
 #define GWC_POINT_HANDLE 0x01
 #define SBW  128	/* Sample Block Width, the number of audio sammples summarized in one block  */
 #define STATUS_UPDATE_INTERVAL 0.5	/* update status bar every 1/2  second on long edit operations */
@@ -56,6 +56,11 @@
 #define REPAIR_CLIPPED 2
 #define REPAIR_FAILURE 3
 #define DETECT_ONLY 4
+#define REPAIR_OOB 5
+
+/* defs for declick detection method */
+#define FFT_DETECT 0
+#define HPF_DETECT 1
 
 /* defs for denoise */
 #define DENOISE_MAX_FFT 32768
@@ -111,6 +116,10 @@ struct denoise_prefs {
     double amount ;
     double dn_gamma ;
     double randomness ;
+    double min_sample_freq ;
+    double max_sample_freq ;
+    int freq_filter ;
+    int estimate_power_floor ;
 } ;
 
 struct view {
@@ -189,11 +198,18 @@ void set_scroll_bar(long n, long first, long last);
 void display_times(void) ;
 char *do_declick(struct sound_prefs *p, long noise_start, long noise_end, int channel_selection_mask, double sensitivity, int repair,
                struct click_data *, int iterate_flag) ;
+char *do_declick_fft(struct sound_prefs *p, long noise_start, long noise_end, int channel_selection_mask, double sensitivity, int repair,
+               struct click_data *, int iterate_flag) ;
+char *do_declick_hpf(struct sound_prefs *p, long noise_start, long noise_end, int channel_selection_mask, double sensitivity, int repair,
+               struct click_data *, int iterate_flag) ;
 int do_decrackle(struct sound_prefs *p, long noise_start, long noise_end, int channel_selection_mask, double level, gint nmax, gint width) ;
+void estimate_region(fftw_real data[], int firstbad, int lastbad, int siglen) ;
 #ifndef TRUNCATE_OLD
 void resize_sample_buffer(struct sound_prefs *p);
 #endif
 void fill_sample_buffer(struct sound_prefs *p) ;
+void filter_audio(struct sound_prefs *p, long first, long last, int channel_mask) ;
+int  filter_dialog(struct sound_prefs current, struct view *) ;
 void flush_wavefile_data(void) ;
 void get_region_of_interest(long *first, long *last, struct view *v) ;
 int get_sample_buffer(struct sample_block **result) ;
@@ -217,6 +233,8 @@ void move_song_marker(void) ;
 void delete_song_marker(void) ;
 void select_song_marker(void) ;
 struct sound_prefs open_wavefile(char *filename, struct view *v) ;
+void pinknoise(struct sound_prefs *p, long first, long last, int channel_mask) ;
+int  pinknoise_dialog(struct sound_prefs current, struct view *) ;
 int  play_wavefile_data(long first, long last) ;
 void pop_status_text(void) ;
 void push_status_text(gchar *msg) ;
@@ -258,7 +276,7 @@ void resample_audio_data(struct sound_prefs *p, long first, long last) ;
 int  start_recording(char *input_device, char *filename) ;
 long start_playback(char *output_device, struct view *v, struct sound_prefs *p, double seconds_per_block, double seconds_to_preload) ;
 int  start_monitor(char *input_device) ;
-void stop_playback(void) ;
+void stop_playback(int force) ;
 void stop_recording(void) ;
 int start_save_undo(char *undo_msg, struct view *v) ;
 /* void sum_sample_block(struct sample_block *sb, double left[], double right[], long n) ; */
