@@ -30,7 +30,7 @@
 
 #ifndef	MATRIXH
 
-/* RCS id: $Id: matrix.h,v 1.1.1.1 2002/09/08 04:03:51 welty Exp $ */
+/* RCS id: $Id: matrix.h,v 1.18 1994/04/16 00:33:37 des Exp $ */
 
 #define	MATRIXH	
 
@@ -39,21 +39,23 @@
 #include 	"meminfo.h"
 
 /* unsigned integer type */
+/************************************************************
 #ifndef U_INT_DEF
 typedef	unsigned int	u_int;
 #define U_INT_DEF
 #endif
+************************************************************/
 
 /* vector definition */
 typedef	struct	{
-		u_int	dim, max_dim;
+		unsigned int	dim, max_dim;
 		Real	*ve;
 		} VEC;
 
 /* matrix definition */
 typedef	struct	{
-		u_int	m, n;
-		u_int	max_m, max_n, max_size;
+		unsigned int	m, n;
+		unsigned int	max_m, max_n, max_size;
 		Real	**me,*base;	/* base is base of alloc'd mem */
 		} MAT;
 
@@ -66,12 +68,12 @@ typedef struct {
 
 /* permutation definition */
 typedef	struct	{
-		u_int	size, max_size, *pe;
+		unsigned int	size, max_size, *pe;
 		} PERM;
 
 /* integer vector definition */
 typedef struct	{
-		u_int	dim, max_dim;
+		unsigned int	dim, max_dim;
 		int	*ive;
 	        } IVEC;
 
@@ -84,7 +86,27 @@ extern	void	*malloc(size_t),
 		*calloc(size_t,size_t),
 		*realloc(void *,size_t);
 #endif
+#endif /* MALLOCDECL */
+
+/* For creating MEX files (for use with Matlab) using Meschach
+   See also: mexmesch.h */
+#ifdef MEX
+#include	"mex.h"
+#define	malloc(len)		mxMalloc(len)
+#define	calloc(n,len)		mxCalloc(n,len)
+#define	realloc(ptr,len)	mxRealloc(ptr,len)
+#define	free(ptr)		mxFree(ptr)
+#define	printf			mexPrintf
+#ifndef THREADSAFE	/* for use as a shared library */
+#define	THREADSAFE 1
 #endif
+#endif /* MEX */
+
+#ifdef THREADSAFE
+#define	STATIC
+#else
+#define	STATIC	static
+#endif /* THREADSAFE */
 
 #ifndef ANSI_C
 extern void m_version();
@@ -94,19 +116,19 @@ void	m_version( void );
 
 #ifndef ANSI_C
 /* allocate one object of given type */
-#define	NEW(type)	((type *)calloc(1,sizeof(type)))
+#define	NEW(type)	((type *)calloc((size_t)1,sizeof(type)))
 
 /* allocate num objects of given type */
-#define	NEW_A(num,type)	((type *)calloc((unsigned)(num),sizeof(type)))
+#define	NEW_A(num,type)	((type *)calloc((size_t)(num),sizeof(type)))
 
  /* re-allocate arry to have num objects of the given type */
 #define	RENEW(var,num,type) \
     ((var)=(type *)((var) ? \
-		    realloc((char *)(var),(unsigned)(num)*sizeof(type)) : \
-		    calloc((unsigned)(num),sizeof(type))))
+		    realloc((char *)(var),(size_t)(num)*sizeof(type)) : \
+		    calloc((size_t)(num),sizeof(type))))
 
 #define	MEMCOPY(from,to,n_items,type) \
-    MEM_COPY((char *)(from),(char *)(to),(unsigned)(n_items)*sizeof(type))
+    MEM_COPY((char *)(from),(char *)(to),(size_t)(n_items)*sizeof(type))
 
 #else
 /* allocate one object of given type */
@@ -124,15 +146,15 @@ void	m_version( void );
 #define	MEMCOPY(from,to,n_items,type) \
  MEM_COPY((char *)(from),(char *)(to),(unsigned)(n_items)*sizeof(type))
 
-#endif
+#endif /* ANSI_C */
 
 /* type independent min and max operations */
 #ifndef max
 #define	max(a,b)	((a) > (b) ? (a) : (b))
-#endif
+#endif /* max */
 #ifndef min
 #define	min(a,b)	((a) > (b) ? (b) : (a))
-#endif
+#endif /* min */
 
 
 #undef TRUE
@@ -181,7 +203,7 @@ extern  int iv_free(IVEC *);
 extern	m_free(MAT *),v_free(VEC *),px_free(PERM *);
 extern   int bd_free(BAND *);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* MACROS */
@@ -192,74 +214,58 @@ extern   int bd_free(BAND *);
 #define	PX_FREE(px)	( px_free(px),	(px)=(PERM *)NULL )
 #define	IV_FREE(iv)	( iv_free(iv),	(iv)=(IVEC *)NULL )
 
-#define MAXDIM  	2001
+#define MAXDIM  	10000001
 
 
 /* Entry level access to data structures */
+/* routines to check indexes */
+#define	m_chk_idx(A,i,j)	((i)>=0 && (i)<(A)->m && (j)>=0 && (j)<=(A)->n)
+#define	v_chk_idx(x,i)		((i)>=0 && (i)<(x)->dim)
+#define	bd_chk_idx(A,i,j)	((i)>=max(0,(j)-(A)->ub) && \
+		(j)>=max(0,(i)-(A)->lb) && (i)<(A)->mat->n && (j)<(A)->mat->n)
+
+#define	m_entry(A,i,j)		m_get_val(A,i,j)
+#define	v_entry(x,i)		v_get_val(x,i)
+#define	bd_entry(A,i,j)		bd_get_val(A,i,j)
 #ifdef DEBUG
-
-/* returns x[i] */
-#define	v_entry(x,i)	(((i) < 0 || (i) >= (x)->dim) ? \
-			 error(E_BOUNDS,"v_entry"), 0.0 : (x)->ve[i] )
-
-/* x[i] <- val */
-#define	v_set_val(x,i,val) ((x)->ve[i] = ((i) < 0 || (i) >= (x)->dim) ? \
-			    error(E_BOUNDS,"v_set_val"), 0.0 : (val))
-
-/* x[i] <- x[i] + val */
-#define	v_add_val(x,i,val) ((x)->ve[i] += ((i) < 0 || (i) >= (x)->dim) ? \
-			    error(E_BOUNDS,"v_add_val"), 0.0 : (val))
-
-/* x[i] <- x[i] - val */
-#define	v_sub_val(x,i,val) ((x)->ve[i] -= ((i) < 0 || (i) >= (x)->dim) ? \
-			    error(E_BOUNDS,"v_sub_val"), 0.0 : (val))
-
-/* returns A[i][j] */
-#define	m_entry(A,i,j)	(((i) < 0 || (i) >= (A)->m || \
-			  (j) < 0 || (j) >= (A)->n) ? \
-			 error(E_BOUNDS,"m_entry"), 0.0 : (A)->me[i][j] )
-
-/* A[i][j] <- val */
-#define	m_set_val(A,i,j,val) ((A)->me[i][j] = ((i) < 0 || (i) >= (A)->m || \
-					       (j) < 0 || (j) >= (A)->n) ? \
-			      error(E_BOUNDS,"m_set_val"), 0.0 : (val) )
-
-/* A[i][j] <- A[i][j] + val */
-#define	m_add_val(A,i,j,val) ((A)->me[i][j] += ((i) < 0 || (i) >= (A)->m || \
-						(j) < 0 || (j) >= (A)->n) ? \
-			      error(E_BOUNDS,"m_add_val"), 0.0 : (val) )
-
-/* A[i][j] <- A[i][j] - val */
-#define	m_sub_val(A,i,j,val) ((A)->me[i][j] -= ((i) < 0 || (i) >= (A)->m || \
-						(j) < 0 || (j) >= (A)->n) ? \
-			      error(E_BOUNDS,"m_sub_val"), 0.0 : (val) )
-#else
-
-/* returns x[i] */
-#define	v_entry(x,i)		((x)->ve[i])
-
-/* x[i] <- val */
-#define	v_set_val(x,i,val)	((x)->ve[i]  = (val))
-
-/* x[i] <- x[i] + val */
-#define	v_add_val(x,i,val)	((x)->ve[i] += (val))
-
- /* x[i] <- x[i] - val */
-#define	v_sub_val(x,i,val)	((x)->ve[i] -= (val))
-
-/* returns A[i][j] */
-#define	m_entry(A,i,j)		((A)->me[i][j])
-
-/* A[i][j] <- val */
-#define	m_set_val(A,i,j,val)	((A)->me[i][j]  = (val) )
-
-/* A[i][j] <- A[i][j] + val */
-#define	m_add_val(A,i,j,val)	((A)->me[i][j] += (val) )
-
-/* A[i][j] <- A[i][j] - val */
-#define	m_sub_val(A,i,j,val)	((A)->me[i][j] -= (val) )
-
-#endif
+#define	m_set_val(A,i,j,val)	( m_chk_idx(A,i,j) ? \
+	(A)->me[(i)][(j)] = (val) : (error(E_BOUNDS,"m_set_val"), 0.0))
+#define	m_add_val(A,i,j,val)	( m_chk_idx(A,i,j) ? \
+	(A)->me[(i)][(j)] += (val) : (error(E_BOUNDS,"m_add_val"), 0.0))
+#define	m_sub_val(A,i,j,val)	( m_chk_idx(A,i,j) ? \
+	(A)->me[(i)][(j)] -= (val) : (error(E_BOUNDS,"m_sub_val"), 0.0))
+#define	m_get_val(A,i,j)	( m_chk_idx(A,i,j) ? \
+	(A)->me[(i)][(j)] : (error(E_BOUNDS,"m_get_val"), 0.0))
+#define	v_set_val(x,i,val)	( v_chk_idx(x,i) ? (x)->ve[(i)] = (val) : \
+	(error(E_BOUNDS,"v_set_val"), 0.0))
+#define	v_add_val(x,i,val)	( v_chk_idx(x,i) ? (x)->ve[(i)] += (val) : \
+	(error(E_BOUNDS,"v_set_val"), 0.0))
+#define	v_sub_val(x,i,val)	( v_chk_idx(x,i) ? (x)->ve[(i)] -= (val) : \
+	(error(E_BOUNDS,"v_set_val"), 0.0))
+#define	v_get_val(x,i)	( v_chk_idx(x,i) ? (x)->ve[(i)] : \
+	(error(E_BOUNDS,"v_get_val"), 0.0))
+#define	bd_set_val(A,i,j,val)	( bd_chk_idx(A,i,j) ? \
+	(A)->mat->me[(A)->lb+(j)-(i)][(j)] = (val) : \
+	(error(E_BOUNDS,"bd_set_val"), 0.0))
+#define	bd_add_val(A,i,j,val)	( bd_chk_idx(A,i,j) ? \
+	(A)->mat->me[(A)->lb+(j)-(i)][(j)] += (val) : \
+	(error(E_BOUNDS,"bd_set_val"), 0.0))
+#define	bd_get_val(A,i,j)	( bd_chk_idx(A,i,j) ? \
+	(A)->mat->me[(A)->lb+(j)-(i)][(j)] : \
+	(error(E_BOUNDS,"bd_get_val"), 0.0))
+#else /* no DEBUG */
+#define	m_set_val(A,i,j,val)	((A)->me[(i)][(j)] = (val))
+#define	m_add_val(A,i,j,val)	((A)->me[(i)][(j)] += (val))
+#define	m_sub_val(A,i,j,val)	((A)->me[(i)][(j)] -= (val))
+#define	m_get_val(A,i,j)	((A)->me[(i)][(j)])
+#define	v_set_val(x,i,val)	((x)->ve[(i)] = (val))
+#define	v_add_val(x,i,val)	((x)->ve[(i)] += (val))
+#define	v_sub_val(x,i,val)	((x)->ve[(i)] -= (val))
+#define	v_get_val(x,i)		((x)->ve[(i)])
+#define	bd_set_val(A,i,j,val)	((A)->mat->me[(A)->lb+(j)-(i)][(j)] = (val))
+#define	bd_add_val(A,i,j,val)	((A)->mat->me[(A)->lb+(j)-(i)][(j)] += (val))
+#define	bd_get_val(A,i,j)	((A)->mat->me[(A)->lb+(j)-(i)][(j)])
+#endif /* DEBUG */
 
 
 /* I/O routines */
@@ -277,13 +283,13 @@ extern	double fin_double();
 #else
 
 /* print x on file fp */
-void v_foutput(FILE *fp,VEC *x),
+void v_foutput(FILE *fp,const VEC *x),
        /* print A on file fp */
-	m_foutput(FILE *fp,MAT *A),
+	m_foutput(FILE *fp,const MAT *A),
        /* print px on file fp */
-	px_foutput(FILE *fp,PERM *px);
+	px_foutput(FILE *fp,const PERM *px);
 /* print ix on file fp */
-void iv_foutput(FILE *fp,IVEC *ix);
+void iv_foutput(FILE *fp,const IVEC *ix);
 
 /* Note: if out is NULL, then returned object is newly allocated;
         Also: if out is not NULL, then that size is assumed */
@@ -300,7 +306,7 @@ IVEC *iv_finput(FILE *fp,IVEC *out);
 /* fy_or_n -- yes-or-no to question in string s
         -- question written to stderr, input from fp 
         -- if fp is NOT a tty then return y_n_dflt */
-int fy_or_n(FILE *fp,char *s);
+int fy_or_n(FILE *fp, const char *s);
 
 /* yn_dflt -- sets the value of y_n_dflt to val */
 int yn_dflt(int val);
@@ -310,20 +316,20 @@ int yn_dflt(int val);
         -- check that x lies between low and high: re-prompt if
                 fp is a tty, error exit otherwise
         -- ignore check if low > high           */
-int fin_int(FILE *fp,char *s,int low,int high);
+int fin_int(FILE *fp,const char *s,int low,int high);
 
 /* fin_double -- return double read from file/stream fp
         -- prompt s on stderr if fp is a tty
         -- check that x lies between low and high: re-prompt if
                 fp is a tty, error exit otherwise
         -- ignore check if low > high           */
-double fin_double(FILE *fp,char *s,double low,double high);
+double fin_double(FILE *fp,const char *s,double low,double high);
 
 /* it skips white spaces and strings of the form #....\n
    Here .... is a comment string */
 int skipjunk(FILE *fp);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* MACROS */
@@ -350,6 +356,9 @@ int skipjunk(FILE *fp);
 #define	in_int(s,lo,hi)	fin_int(stdin,s,lo,hi)
 #define	in_double(s,lo,hi)	fin_double(stdin,s,lo,hi)
 
+
+/* special purpose access routines */
+
 /* Copying routines */
 #ifndef ANSI_C
 extern	MAT	*_m_copy(), *m_move(), *vm_move();
@@ -361,19 +370,19 @@ extern  BAND    *bd_copy();
 #else
 
 /* copy in to out starting at out[i0][j0] */
-extern	MAT	*_m_copy(MAT *in,MAT *out,u_int i0,u_int j0),
-		* m_move(MAT *in, int, int, int, int, MAT *out, int, int),
-		*vm_move(VEC *in, int, MAT *out, int, int, int, int);
+extern	MAT	*_m_copy(const MAT *in,MAT *out,unsigned int i0,unsigned int j0),
+		* m_move(const MAT *in, int, int, int, int, MAT *out, int, int),
+		*vm_move(const VEC *in, int, MAT *out, int, int, int, int);
 /* copy in to out starting at out[i0] */
-extern	VEC	*_v_copy(VEC *in,VEC *out,u_int i0),
-		* v_move(VEC *in, int, int, VEC *out, int),
-		*mv_move(MAT *in, int, int, int, int, VEC *out, int);
-extern	PERM	*px_copy(PERM *in,PERM *out);
-extern	IVEC	*iv_copy(IVEC *in,IVEC *out),
-		*iv_move(IVEC *in, int, int, IVEC *out, int);
-extern  BAND    *bd_copy(BAND *in,BAND *out);
+extern	VEC	*_v_copy(const VEC *in,VEC *out,unsigned int i0),
+		* v_move(const VEC *in, int, int, VEC *out, int),
+		*mv_move(const MAT *in, int, int, int, int, VEC *out, int);
+extern	PERM	*px_copy(const PERM *in,PERM *out);
+extern	IVEC	*iv_copy(const IVEC *in,IVEC *out),
+		*iv_move(const IVEC *in, int, int, IVEC *out, int);
+extern  BAND    *bd_copy(const BAND *in,BAND *out);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* MACROS */
@@ -393,7 +402,7 @@ extern	MAT     *m_zero(MAT *), *m_ident(MAT *), *m_rand(MAT *),
 						*m_ones(MAT *);
 extern	PERM    *px_ident(PERM *);
 extern  IVEC    *iv_zero(IVEC *);
-#endif
+#endif /* ANSI_C */
 
 /* Basic vector operations */
 #ifndef ANSI_C
@@ -407,54 +416,54 @@ extern	void	__mltadd__(), __add__(), __sub__(),
                 __smlt__(), __zero__();
 #else
 
-extern	VEC	*sv_mlt(double,VEC *,VEC *),	/* out <- s.x */
-		*mv_mlt(MAT *,VEC *,VEC *),	/* out <- A.x */
-		*vm_mlt(MAT *,VEC *,VEC *),	/* out^T <- x^T.A */
-		*v_add(VEC *,VEC *,VEC *), 	/* out <- x + y */
-                *v_sub(VEC *,VEC *,VEC *),	/* out <- x - y */
-		*px_vec(PERM *,VEC *,VEC *),	/* out <- P.x */
-		*pxinv_vec(PERM *,VEC *,VEC *),	  /* out <- P^{-1}.x */
-		*v_mltadd(VEC *,VEC *,double,VEC *),   /* out <- x + s.y */
+extern	VEC	*sv_mlt(double s,const VEC *x,VEC *out),	/* out <- s.x */
+		*mv_mlt(const MAT *A,const VEC *s,VEC *out),	/* out <- A.x */
+		*vm_mlt(const MAT *A,const VEC *x,VEC *out),	/* out^T <- x^T.A */
+		*v_add(const VEC *x,const VEC *y,VEC *out), 	/* out <- x + y */
+                *v_sub(const VEC *x,const VEC *y,VEC *out),	/* out <- x - y */
+		*px_vec(PERM *px,const VEC *x,VEC *out),	/* out <- P.x */
+		*pxinv_vec(PERM *px,const VEC *x,VEC *out),	/* out <- P^{-1}.x */
+		*v_mltadd(const VEC *x,const VEC *y,double s,VEC *out),   /* out <- x + s.y */
 #ifdef PROTOTYPES_IN_STRUCT
-		*v_map(double (*f)(double),VEC *,VEC *),  
+		*v_map(double (*f)(double),const VEC *x,VEC *y),  
                                                  /* out[i] <- f(x[i]) */
-		*_v_map(double (*f)(void *,double),void *,VEC *,VEC *),
+		*_v_map(double (*f)(void *,double),void *p,const VEC *x,VEC *y),
 #else
-		*v_map(double (*f)(),VEC *,VEC *), /* out[i] <- f(x[i]) */
-		*_v_map(double (*f)(),void *,VEC *,VEC *),
-#endif
-		*v_lincomb(int,VEC **,Real *,VEC *),   
+		*v_map(double (*f)(),const VEC *,VEC *), /* out[i] <- f(x[i]) */
+		*_v_map(double (*f)(),void *,const VEC *,VEC *),
+#endif /* PROTOTYPES_IN_STRUCT */
+		*v_lincomb(int,const VEC **,const Real *,VEC *),   
                                                  /* out <- sum_i s[i].x[i] */
                 *v_linlist(VEC *out,VEC *v1,double a1,...);
                                               /* out <- s1.x1 + s2.x2 + ... */
 
 /* returns min_j x[j] (== x[i]) */
-extern	double	v_min(VEC *, int *), 
+extern	double	v_min(const VEC *, int *), 
      /* returns max_j x[j] (== x[i]) */		
-        v_max(VEC *, int *), 
+        v_max(const VEC *, int *), 
         /* returns sum_i x[i] */
-        v_sum(VEC *);
+        v_sum(const VEC *);
 
 /* Hadamard product: out[i] <- x[i].y[i] */
-extern	VEC	*v_star(VEC *, VEC *, VEC *),
+extern	VEC	*v_star(const VEC *, const VEC *, VEC *),
                  /* out[i] <- x[i] / y[i] */
-		*v_slash(VEC *, VEC *, VEC *),
+		*v_slash(const VEC *, const VEC *, VEC *),
                /* sorts x, and sets order so that sorted x[i] = x[order[i]] */ 
 		*v_sort(VEC *, PERM *);
 
 /* returns inner product starting at component i0 */
-extern	double	_in_prod(VEC *x,VEC *y,u_int i0), 
+extern	double	_in_prod(const VEC *x, const VEC *y,unsigned int i0),
                 /* returns sum_{i=0}^{len-1} x[i].y[i] */
-                __ip__(Real *,Real *,int);
+                __ip__(const Real *,const Real *,int);
 
 /* see v_mltadd(), v_add(), v_sub() and v_zero() */
-extern	void	__mltadd__(Real *,Real *,double,int),
-		__add__(Real *,Real *,Real *,int),
-		__sub__(Real *,Real *,Real *,int),
-                __smlt__(Real *,double,Real *,int),
+extern	void	__mltadd__(Real *,const Real *,double,int),
+		__add__(const Real *,const Real *,Real *,int),
+		__sub__(const Real *,const Real *,Real *,int),
+                __smlt__(const Real *,double,Real *,int),
 		__zero__(Real *,int);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* MACRO */
@@ -470,16 +479,18 @@ extern	double	_v_norm1(), _v_norm2(), _v_norm_inf(),
 
 #else
                /* returns sum_i |x[i]/scale[i]| */
-extern	double	_v_norm1(VEC *x,VEC *scale),   
+extern	double	_v_norm1(const VEC *x,const VEC *scale),   
                /* returns (scaled) Euclidean norm */
-                _v_norm2(VEC *x,VEC *scale),
+                _v_norm2(const VEC *x,const VEC *scale),
                /* returns max_i |x[i]/scale[i]| */
-		_v_norm_inf(VEC *x,VEC *scale);
+		_v_norm_inf(const VEC *x,const VEC *scale);
 
 /* unscaled matrix norms */
-extern double m_norm1(MAT *A), m_norm_inf(MAT *A), m_norm_frob(MAT *A);
+extern double m_norm1(const MAT *A), 
+	m_norm_inf(const MAT *A), 
+	m_norm_frob(const MAT *A);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* MACROS */
@@ -494,44 +505,55 @@ extern double m_norm1(MAT *A), m_norm_inf(MAT *A), m_norm_frob(MAT *A);
 extern	MAT *sm_mlt(), *m_mlt(), *mmtr_mlt(), *mtrm_mlt(), *m_add(), *m_sub(),
 		*sub_mat(), *m_transp(), *ms_mltadd();
 
-extern   BAND *bd_transp();
+extern  BAND *bd_transp(), *sbd_mlt(), *bds_mltadd(), *bd_zero();
 extern	MAT *px_rows(), *px_cols(), *swap_rows(), *swap_cols(),
              *_set_row(), *_set_col();
 extern	VEC *get_row(), *get_col(), *sub_vec(),
-		*mv_mltadd(), *vm_mltadd();
+		*mv_mltadd(), *vm_mltadd(), *bdv_mltadd();
 
 #else
 
-extern	MAT	*sm_mlt(double s,MAT *A,MAT *out), 	/* out <- s.A */
-		*m_mlt(MAT *A,MAT *B,MAT *out),	/* out <- A.B */
-		*mmtr_mlt(MAT *A,MAT *B,MAT *out),	/* out <- A.B^T */
-		*mtrm_mlt(MAT *A,MAT *B,MAT *out),	/* out <- A^T.B */
-		*m_add(MAT *A,MAT *B,MAT *out),	/* out <- A + B */
-		*m_sub(MAT *A,MAT *B,MAT *out),	/* out <- A - B */
-		*sub_mat(MAT *A,u_int,u_int,u_int,u_int,MAT *out),
-		*m_transp(MAT *A,MAT *out),		/* out <- A^T */
+extern	MAT	*sm_mlt(double s, const MAT *A,MAT *out), 	/* out <- s.A */
+		*m_mlt(const MAT *A,const MAT *B,MAT *out),	/* out <- A.B */
+		*mmtr_mlt(const MAT *A,const MAT *B,MAT *out),	/* out <- A.B^T */
+		*mtrm_mlt(const MAT *A,const MAT *B,MAT *out),	/* out <- A^T.B */
+		*m_add(const MAT *A,const MAT *B,MAT *out),	/* out <- A + B */
+		*m_sub(const MAT *A,const MAT *B,MAT *out),	/* out <- A - B */
+		*sub_mat(const MAT *A,unsigned int,unsigned int,unsigned int,
+			 unsigned int,MAT *out),
+		*m_transp(const MAT *A,MAT *out),		/* out <- A^T */
                 /* out <- A + s.B */ 
-		*ms_mltadd(MAT *A,MAT *B,double s,MAT *out);   
+		*ms_mltadd(const MAT *A,const MAT *B,double s,MAT *out);   
 
 
-extern  BAND    *bd_transp(BAND *in, BAND *out);   /* out <- A^T */
-extern	MAT	*px_rows(PERM *px,MAT *A,MAT *out),	/* out <- P.A */
-		*px_cols(PERM *px,MAT *A,MAT *out),	/* out <- A.P^T */
+extern  BAND    *bd_transp(const BAND *in, BAND *out),	/* out <- A^T */
+  *sbd_mlt(Real s, const BAND *A, BAND *OUT),		/* OUT <- s.A */
+  *bds_mltadd(const BAND *A, const BAND *B,double alpha, BAND *OUT),
+  /* OUT <- A+alpha.B */
+  *bd_zero(BAND *A);					/* A <- 0 */
+
+extern	MAT	*px_rows(const PERM *px,const MAT *A,MAT *out),	/* out <- P.A */
+		*px_cols(const PERM *px,const MAT *A,MAT *out),	/* out <- A.P^T */
 		*swap_rows(MAT *,int,int,int,int),
 		*swap_cols(MAT *,int,int,int,int),
                  /* A[i][j] <- out[j], j >= j0 */
-		*_set_col(MAT *A,u_int i,VEC *out,u_int j0),
+		*_set_col(MAT *A,unsigned int i,const VEC *col,unsigned int j0),
                  /* A[i][j] <- out[i], i >= i0 */
-		*_set_row(MAT *A,u_int j,VEC *out,u_int i0);
+		*_set_row(MAT *A,unsigned int j,const VEC *row,unsigned int i0);
 
-extern	VEC	*get_row(MAT *,u_int,VEC *),
-		*get_col(MAT *,u_int,VEC *),
-		*sub_vec(VEC *,int,int,VEC *),
-                   /* out <- x + s.A.y */
-		*mv_mltadd(VEC *x,VEC *y,MAT *A,double s,VEC *out),
-                  /* out^T <- x^T + s.y^T.A */
-		*vm_mltadd(VEC *x,VEC *y,MAT *A,double s,VEC *out);
-#endif
+extern	VEC	*get_row(const MAT *,unsigned int,VEC *),
+		*get_col(const MAT *,unsigned int,VEC *),
+		*sub_vec(const VEC *,int,int,VEC *),
+                   /* mv_mltadd: out <- x + s.A.y */
+		*mv_mltadd(const VEC *x,const VEC *y,const MAT *A,
+			   double s,VEC *out),
+                  /* vm_mltadd: out^T <- x^T + s.y^T.A */
+		*vm_mltadd(const VEC *x,const VEC *y,const MAT *A,
+			   double s,VEC *out),
+                  /* bdv_mltadd: out <- x + s.A.y */
+                *bdv_mltadd(const VEC *x,const VEC *y,const BAND *A,
+			    double s,VEC *out);
+#endif /* ANSI_C */
 
 
 /* MACROS */
@@ -549,16 +571,16 @@ extern	int  px_sign();
 
 #else
 
-extern	PERM	*px_mlt(PERM *px1,PERM *px2,PERM *out),	/* out <- px1.px2 */
-		*px_inv(PERM *px,PERM *out),	/* out <- px^{-1} */
+extern	PERM	*px_mlt(const PERM *px1,const PERM *px2,PERM *out),	/* out <- px1.px2 */
+		*px_inv(const PERM *px,PERM *out),	/* out <- px^{-1} */
                  /* swap px[i] and px[j] */
-		*px_transp(PERM *px,u_int i,u_int j);
+		*px_transp(PERM *px,unsigned int i,unsigned int j);
 
      /* returns sign(px) = +1 if px product of even # transpositions
                            -1 if ps product of odd  # transpositions */
-extern	int	px_sign(PERM *);
+extern	int	px_sign(const PERM *);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* Basic integer vector operations */
@@ -568,12 +590,14 @@ extern	IVEC	*iv_add(), *iv_sub(), *iv_sort();
 
 #else
 
-extern	IVEC	*iv_add(IVEC *ix,IVEC *iy,IVEC *out),  /* out <- ix + iy */
-		*iv_sub(IVEC *ix,IVEC *iy,IVEC *out),  /* out <- ix - iy */
-        /* sorts ix & sets order so that sorted ix[i] = old ix[order[i]] */
+extern	IVEC	*iv_add(const IVEC *ix,const IVEC *iy,IVEC *out),  
+  /* out <- ix + iy */
+		*iv_sub(const IVEC *ix,const IVEC *iy,IVEC *out),  
+  /* out <- ix - iy */
+  /* sorts ix & sets order so that sorted ix[i] = old ix[order[i]] */
 		*iv_sort(IVEC *ix, PERM *order);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* miscellaneous functions */
@@ -595,13 +619,13 @@ double	square(double x), 	/* returns x^2 */
 void	smrand(int seed),            /* seeds mrand() */
   mrandlist(Real *x, int len);       /* generates len random numbers */
 
-void    m_dump(FILE *fp,MAT *a), px_dump(FILE *,PERM *px),
-        v_dump(FILE *fp,VEC *x), iv_dump(FILE *fp, IVEC *ix);
+void    m_dump(FILE *fp,const MAT *a), px_dump(FILE *fp, const PERM *px),
+        v_dump(FILE *fp,const VEC *x), iv_dump(FILE *fp, const IVEC *ix);
 
-MAT *band2mat(BAND *bA, MAT *A);
-BAND *mat2band(MAT *A, int lb,int ub, BAND *bA);
+MAT *band2mat(const BAND *bA, MAT *A);
+BAND *mat2band(const MAT *A, int lb,int ub, BAND *bA);
 
-#endif
+#endif /* ANSI_C */
 
 
 /* miscellaneous constants */
@@ -657,9 +681,9 @@ int iv_free_vars();
 int px_free_vars();
 int m_free_vars();
 
-#endif
+#endif /* ANSI_C */
 
 
-#endif
+#endif /* MATRIXH */
 
 
