@@ -718,13 +718,20 @@ void declick_set_preferences(GtkWidget * widget, gpointer data)
 {
     extern double weak_declick_sensitivity;
     extern double strong_declick_sensitivity;
-    extern int declick_iterate_flag;
+    extern double weak_fft_declick_sensitivity;
+    extern double strong_fft_declick_sensitivity;
+    extern int declick_iterate_flag ;
+    extern int declick_detector_type ;
     GtkWidget *dlg;
     GtkWidget *dc_weak_entry;
     GtkWidget *dc_strong_entry;
-    GtkWidget *dialog_table;
     GtkWidget *iterate_widget;
+    GtkWidget *dc_fft_weak_entry;
+    GtkWidget *dc_fft_strong_entry;
+    GtkWidget *method_widget;
+    GtkWidget *dialog_table;
     int dres;
+    int row = 1 ;
 
     dlg =
 	gtk_dialog_new_with_buttons("Declicking preferences",
@@ -742,19 +749,33 @@ void declick_set_preferences(GtkWidget * widget, gpointer data)
     dc_weak_entry =
 	add_number_entry_with_label_double(weak_declick_sensitivity,
 					   "Weak Declick Sensitivity (default = 1.0) ",
-					   dialog_table, 1);
+					   dialog_table, row++);
     dc_strong_entry =
 	add_number_entry_with_label_double(strong_declick_sensitivity,
 					   "Strong Declick Sensitivity (default = 0.75) ",
-					   dialog_table, 2);
-    iterate_widget =
-	gtk_check_button_new_with_label
-	("Iterate in repair clicks until all repaired");
-    gtk_toggle_button_set_active((GtkToggleButton *) iterate_widget,
-				 declick_iterate_flag == 1 ? TRUE : FALSE);
+					   dialog_table, row++);
+
+    dc_fft_weak_entry =
+	add_number_entry_with_label_double(weak_fft_declick_sensitivity,
+					   "FFT Weak Declick Sensitivity (default = 3.0) ",
+					   dialog_table, row++);
+    dc_fft_strong_entry =
+	add_number_entry_with_label_double(strong_fft_declick_sensitivity,
+					   "FFT Strong Declick Sensitivity (default = 5.0) ",
+					   dialog_table, row++);
+
+    method_widget = gtk_check_button_new_with_label ("Use FFT click detector");
+    gtk_toggle_button_set_active((GtkToggleButton *) method_widget, declick_detector_type == FFT_DETECT ? TRUE : FALSE);
+    gtk_widget_show(method_widget);
+    gtk_table_attach_defaults(GTK_TABLE(dialog_table), method_widget, 0, 2, row, row+1);
+    row += 2 ;
+
+    iterate_widget = gtk_check_button_new_with_label ("Iterate in repair clicks until all repaired");
+    gtk_toggle_button_set_active((GtkToggleButton *) iterate_widget, declick_iterate_flag == 1 ? TRUE : FALSE);
     gtk_widget_show(iterate_widget);
-    gtk_table_attach_defaults(GTK_TABLE(dialog_table), iterate_widget, 0,
-			      2, 3, 4);
+    gtk_table_attach_defaults(GTK_TABLE(dialog_table), iterate_widget, 0, 2, row, row+1);
+    row += 2 ;
+
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), dialog_table,
 		       TRUE, TRUE, 0);
@@ -762,14 +783,14 @@ void declick_set_preferences(GtkWidget * widget, gpointer data)
     dres = gwc_dialog_run(GTK_DIALOG(dlg));
 
     if (dres == 0) {
-	weak_declick_sensitivity =
-	    atof(gtk_entry_get_text((GtkEntry *) dc_weak_entry));
-	strong_declick_sensitivity =
-	    atof(gtk_entry_get_text((GtkEntry *) dc_strong_entry));
-	declick_iterate_flag =
-	    gtk_toggle_button_get_active((GtkToggleButton *)
-					 iterate_widget) ==
+	weak_declick_sensitivity = atof(gtk_entry_get_text((GtkEntry *) dc_weak_entry));
+	strong_declick_sensitivity = atof(gtk_entry_get_text((GtkEntry *) dc_strong_entry));
+	weak_fft_declick_sensitivity = atof(gtk_entry_get_text((GtkEntry *) dc_fft_weak_entry));
+	strong_fft_declick_sensitivity = atof(gtk_entry_get_text((GtkEntry *) dc_fft_strong_entry));
+	declick_iterate_flag = gtk_toggle_button_get_active((GtkToggleButton *) iterate_widget) ==
 	    TRUE ? 1 : 0;
+	declick_detector_type = gtk_toggle_button_get_active((GtkToggleButton *) method_widget) ==
+	    TRUE ? FFT_DETECT : HPF_DETECT ;
     }
 
     gtk_widget_destroy(dlg) ;
@@ -985,7 +1006,7 @@ static int noise_suppression_method, window_type;
 
 void load_denoise_preferences(void)
 {
-    gnome_config_push_prefix("/gnome_wave_cleaner/denoise_params/");
+    gnome_config_push_prefix(APPNAME"/denoise_params/");
     denoise_prefs.n_noise_samples =
 	gnome_config_get_int("n_noise_samples=16");
     denoise_prefs.smoothness = gnome_config_get_int("smoothness=11");
@@ -994,6 +1015,10 @@ void load_denoise_preferences(void)
     denoise_prefs.dn_gamma = gnome_config_get_float("dn_gamma=0.95");
     denoise_prefs.randomness = gnome_config_get_float("randomness=0.0");
     denoise_prefs.window_type = gnome_config_get_int("window_type=1");
+    denoise_prefs.freq_filter = gnome_config_get_int("freq_filter=0");
+    denoise_prefs.estimate_power_floor = gnome_config_get_int("estimate_power_floor=0");
+    denoise_prefs.min_sample_freq = gnome_config_get_float("min_sample_freq=0.0");
+    denoise_prefs.max_sample_freq = gnome_config_get_float("max_sample_freq=44100.0");
     denoise_prefs.noise_suppression_method =
 	gnome_config_get_int("noise_suppression_method=1");
     gnome_config_pop_prefix();
@@ -1001,7 +1026,7 @@ void load_denoise_preferences(void)
 
 void save_denoise_preferences(void)
 {
-    gnome_config_push_prefix("/gnome_wave_cleaner/denoise_params/");
+    gnome_config_push_prefix(APPNAME"/denoise_params/");
     gnome_config_set_int("n_noise_samples", denoise_prefs.n_noise_samples);
     gnome_config_set_int("smoothness", denoise_prefs.smoothness);
     gnome_config_set_int("FFT_SIZE", denoise_prefs.FFT_SIZE);
@@ -1009,6 +1034,12 @@ void save_denoise_preferences(void)
     gnome_config_set_float("dn_gamma", denoise_prefs.dn_gamma);
     gnome_config_set_float("randomness", denoise_prefs.randomness);
     gnome_config_set_int("window_type", denoise_prefs.window_type);
+
+    gnome_config_set_int("freq_filter", denoise_prefs.freq_filter);
+    gnome_config_set_int("estimate_power_floor", denoise_prefs.estimate_power_floor);
+    gnome_config_set_float("min_sample_freq", denoise_prefs.min_sample_freq);
+    gnome_config_set_float("max_sample_freq", denoise_prefs.max_sample_freq);
+
     gnome_config_set_int("noise_suppression_method",
 			 denoise_prefs.noise_suppression_method);
     gnome_config_sync();
@@ -1051,6 +1082,11 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
     GtkWidget *smoothness_entry;
     GtkWidget *n_noise_entry;
     GtkWidget *dialog_table;
+    GtkWidget *freq_filter_entry;
+    GtkWidget *estimate_power_floor_entry;
+    GtkWidget *min_sample_freq_entry;
+    GtkWidget *max_sample_freq_entry;
+
     int dres;
 
     GtkWidget *fft_window_list;
@@ -1111,7 +1147,7 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL, NULL);
 
-    dialog_table = gtk_table_new(5, 2, 0);
+    dialog_table = gtk_table_new(9, 2, 0);
 
     gtk_table_set_row_spacings(GTK_TABLE(dialog_table), 4);
     gtk_table_set_col_spacings(GTK_TABLE(dialog_table), 6);
@@ -1133,11 +1169,28 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 	add_number_entry_with_label_int(denoise_prefs.n_noise_samples,
 					"# noise samples (2-16)",
 					dialog_table, 3);
+
     gamma_entry =
 	add_number_entry_with_label_double(denoise_prefs.dn_gamma,
 					   "gamma -- for Lorber & Hoelrich and Ephraim-Malah , (0-1, try 0.98)",
 					   dialog_table, 4);
 
+    freq_filter_entry =
+	add_number_entry_with_label_int(denoise_prefs.freq_filter,
+					"Apply freq filter (0,1)", dialog_table, 5);
+
+    estimate_power_floor_entry =
+	add_number_entry_with_label_int(denoise_prefs.estimate_power_floor,
+					"Estimate power floor (0,1)", dialog_table, 6);
+
+    min_sample_freq_entry =
+	add_number_entry_with_label_int(denoise_prefs.min_sample_freq,
+					"Minimum frequency to use in noise sample (hz)", dialog_table, 7);
+
+    max_sample_freq_entry =
+	add_number_entry_with_label_int(denoise_prefs.max_sample_freq,
+					"Maximum frequency to use in noise sample (hz)", dialog_table, 8);
+
 
 /*      combo_entry1 = gnome_number_entry_gtk_entry (GNOME_NUMBER_ENTRY (numberentry1));  */
 /*      gtk_widget_show (combo_entry1);  */
@@ -1169,6 +1222,10 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 	denoise_prefs.noise_suppression_method =
 	    noise_suppression_method;
 	denoise_prefs.window_type = window_type;
+	denoise_prefs.freq_filter = atoi(gtk_entry_get_text((GtkEntry *) freq_filter_entry));
+	denoise_prefs.estimate_power_floor = atoi(gtk_entry_get_text((GtkEntry *) estimate_power_floor_entry));
+	denoise_prefs.min_sample_freq = atof(gtk_entry_get_text((GtkEntry *) min_sample_freq_entry));
+	denoise_prefs.max_sample_freq = atof(gtk_entry_get_text((GtkEntry *) max_sample_freq_entry));
 	save_denoise_preferences();
     }
 
